'''OpenGL extension NV.fragment_program

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_NV_fragment_program'
_DEPRECATED = False
GL_MAX_FRAGMENT_PROGRAM_LOCAL_PARAMETERS_NV = constant.Constant( 'GL_MAX_FRAGMENT_PROGRAM_LOCAL_PARAMETERS_NV', 0x8868 )
glget.addGLGetConstant( GL_MAX_FRAGMENT_PROGRAM_LOCAL_PARAMETERS_NV, (1,) )
GL_FRAGMENT_PROGRAM_NV = constant.Constant( 'GL_FRAGMENT_PROGRAM_NV', 0x8870 )
glget.addGLGetConstant( GL_FRAGMENT_PROGRAM_NV, (1,) )
GL_MAX_TEXTURE_COORDS_NV = constant.Constant( 'GL_MAX_TEXTURE_COORDS_NV', 0x8871 )
glget.addGLGetConstant( GL_MAX_TEXTURE_COORDS_NV, (1,) )
GL_MAX_TEXTURE_IMAGE_UNITS_NV = constant.Constant( 'GL_MAX_TEXTURE_IMAGE_UNITS_NV', 0x8872 )
glget.addGLGetConstant( GL_MAX_TEXTURE_IMAGE_UNITS_NV, (1,) )
GL_FRAGMENT_PROGRAM_BINDING_NV = constant.Constant( 'GL_FRAGMENT_PROGRAM_BINDING_NV', 0x8873 )
glget.addGLGetConstant( GL_FRAGMENT_PROGRAM_BINDING_NV, (1,) )
GL_PROGRAM_ERROR_STRING_NV = constant.Constant( 'GL_PROGRAM_ERROR_STRING_NV', 0x8874 )
glProgramNamedParameter4fNV = platform.createExtensionFunction( 
'glProgramNamedParameter4fNV',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLsizei,arrays.GLubyteArray,constants.GLfloat,constants.GLfloat,constants.GLfloat,constants.GLfloat,),
doc='glProgramNamedParameter4fNV(GLuint(id), GLsizei(len), GLubyteArray(name), GLfloat(x), GLfloat(y), GLfloat(z), GLfloat(w)) -> None',
argNames=('id','len','name','x','y','z','w',),
deprecated=_DEPRECATED,
)

glProgramNamedParameter4dNV = platform.createExtensionFunction( 
'glProgramNamedParameter4dNV',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLsizei,arrays.GLubyteArray,constants.GLdouble,constants.GLdouble,constants.GLdouble,constants.GLdouble,),
doc='glProgramNamedParameter4dNV(GLuint(id), GLsizei(len), GLubyteArray(name), GLdouble(x), GLdouble(y), GLdouble(z), GLdouble(w)) -> None',
argNames=('id','len','name','x','y','z','w',),
deprecated=_DEPRECATED,
)

glProgramNamedParameter4fvNV = platform.createExtensionFunction( 
'glProgramNamedParameter4fvNV',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLsizei,arrays.GLubyteArray,arrays.GLfloatArray,),
doc='glProgramNamedParameter4fvNV(GLuint(id), GLsizei(len), GLubyteArray(name), GLfloatArray(v)) -> None',
argNames=('id','len','name','v',),
deprecated=_DEPRECATED,
)

glProgramNamedParameter4dvNV = platform.createExtensionFunction( 
'glProgramNamedParameter4dvNV',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLsizei,arrays.GLubyteArray,arrays.GLdoubleArray,),
doc='glProgramNamedParameter4dvNV(GLuint(id), GLsizei(len), GLubyteArray(name), GLdoubleArray(v)) -> None',
argNames=('id','len','name','v',),
deprecated=_DEPRECATED,
)

glGetProgramNamedParameterfvNV = platform.createExtensionFunction( 
'glGetProgramNamedParameterfvNV',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLsizei,arrays.GLubyteArray,arrays.GLfloatArray,),
doc='glGetProgramNamedParameterfvNV(GLuint(id), GLsizei(len), GLubyteArray(name), GLfloatArray(params)) -> None',
argNames=('id','len','name','params',),
deprecated=_DEPRECATED,
)

glGetProgramNamedParameterdvNV = platform.createExtensionFunction( 
'glGetProgramNamedParameterdvNV',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLsizei,arrays.GLubyteArray,arrays.GLdoubleArray,),
doc='glGetProgramNamedParameterdvNV(GLuint(id), GLsizei(len), GLubyteArray(name), GLdoubleArray(params)) -> None',
argNames=('id','len','name','params',),
deprecated=_DEPRECATED,
)


def glInitFragmentProgramNV():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension EXT.texture_object

This module customises the behaviour of the 
OpenGL.raw.GL.EXT.texture_object to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension introduces named texture objects.  The only way to name
	a texture in GL 1.0 is by defining it as a single display list.  Because
	display lists cannot be edited, these objects are static.  Yet it is
	important to be able to change the images and parameters of a texture.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/texture_object.txt
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions, wrapper
from OpenGL.GL import glget
import ctypes
from OpenGL.raw.GL.EXT.texture_object import *
### END AUTOGENERATED SECTION


'''OpenGL extension VERSION.GL_1_4

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_VERSION_GL_1_4'
_DEPRECATED = False
GL_BLEND_DST_RGB = constant.Constant( 'GL_BLEND_DST_RGB', 0x80C8 )
GL_BLEND_SRC_RGB = constant.Constant( 'GL_BLEND_SRC_RGB', 0x80C9 )
GL_BLEND_DST_ALPHA = constant.Constant( 'GL_BLEND_DST_ALPHA', 0x80CA )
GL_BLEND_SRC_ALPHA = constant.Constant( 'GL_BLEND_SRC_ALPHA', 0x80CB )
GL_POINT_FADE_THRESHOLD_SIZE = constant.Constant( 'GL_POINT_FADE_THRESHOLD_SIZE', 0x8128 )
GL_DEPTH_COMPONENT16 = constant.Constant( 'GL_DEPTH_COMPONENT16', 0x81A5 )
GL_DEPTH_COMPONENT24 = constant.Constant( 'GL_DEPTH_COMPONENT24', 0x81A6 )
GL_DEPTH_COMPONENT32 = constant.Constant( 'GL_DEPTH_COMPONENT32', 0x81A7 )
GL_MIRRORED_REPEAT = constant.Constant( 'GL_MIRRORED_REPEAT', 0x8370 )
GL_MAX_TEXTURE_LOD_BIAS = constant.Constant( 'GL_MAX_TEXTURE_LOD_BIAS', 0x84FD )
GL_TEXTURE_LOD_BIAS = constant.Constant( 'GL_TEXTURE_LOD_BIAS', 0x8501 )
GL_INCR_WRAP = constant.Constant( 'GL_INCR_WRAP', 0x8507 )
GL_DECR_WRAP = constant.Constant( 'GL_DECR_WRAP', 0x8508 )
GL_TEXTURE_DEPTH_SIZE = constant.Constant( 'GL_TEXTURE_DEPTH_SIZE', 0x884A )
GL_TEXTURE_COMPARE_MODE = constant.Constant( 'GL_TEXTURE_COMPARE_MODE', 0x884C )
GL_TEXTURE_COMPARE_FUNC = constant.Constant( 'GL_TEXTURE_COMPARE_FUNC', 0x884D )
glBlendFuncSeparate = platform.createExtensionFunction( 
'glBlendFuncSeparate',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,constants.GLenum,constants.GLenum,),
doc='glBlendFuncSeparate(GLenum(sfactorRGB), GLenum(dfactorRGB), GLenum(sfactorAlpha), GLenum(dfactorAlpha)) -> None',
argNames=('sfactorRGB','dfactorRGB','sfactorAlpha','dfactorAlpha',),
deprecated=_DEPRECATED,
)

glMultiDrawArrays = platform.createExtensionFunction( 
'glMultiDrawArrays',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,arrays.GLintArray,arrays.GLsizeiArray,constants.GLsizei,),
doc='glMultiDrawArrays(GLenum(mode), GLintArray(first), GLsizeiArray(count), GLsizei(primcount)) -> None',
argNames=('mode','first','count','primcount',),
deprecated=_DEPRECATED,
)

glMultiDrawElements = platform.createExtensionFunction( 
'glMultiDrawElements',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,arrays.GLsizeiArray,constants.GLenum,ctypes.POINTER(ctypes.c_void_p),constants.GLsizei,),
doc='glMultiDrawElements(GLenum(mode), GLsizeiArray(count), GLenum(type), POINTER(ctypes.c_void_p)(indices), GLsizei(primcount)) -> None',
argNames=('mode','count','type','indices','primcount',),
deprecated=_DEPRECATED,
)

glPointParameterf = platform.createExtensionFunction( 
'glPointParameterf',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLfloat,),
doc='glPointParameterf(GLenum(pname), GLfloat(param)) -> None',
argNames=('pname','param',),
deprecated=_DEPRECATED,
)

glPointParameterfv = platform.createExtensionFunction( 
'glPointParameterfv',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,arrays.GLfloatArray,),
doc='glPointParameterfv(GLenum(pname), GLfloatArray(params)) -> None',
argNames=('pname','params',),
deprecated=_DEPRECATED,
)

glPointParameteri = platform.createExtensionFunction( 
'glPointParameteri',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLint,),
doc='glPointParameteri(GLenum(pname), GLint(param)) -> None',
argNames=('pname','param',),
deprecated=_DEPRECATED,
)

glPointParameteriv = platform.createExtensionFunction( 
'glPointParameteriv',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,arrays.GLintArray,),
doc='glPointParameteriv(GLenum(pname), GLintArray(params)) -> None',
argNames=('pname','params',),
deprecated=_DEPRECATED,
)
# import legacy entry points to allow checking for bool(entryPoint)
from OpenGL.raw.GL.VERSION.GL_1_4_DEPRECATED import *



'''OpenGL extension EXT.framebuffer_multisample

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_framebuffer_multisample'
_DEPRECATED = False
GL_RENDERBUFFER_SAMPLES_EXT = constant.Constant( 'GL_RENDERBUFFER_SAMPLES_EXT', 0x8CAB )
GL_FRAMEBUFFER_INCOMPLETE_MULTISAMPLE_EXT = constant.Constant( 'GL_FRAMEBUFFER_INCOMPLETE_MULTISAMPLE_EXT', 0x8D56 )
GL_MAX_SAMPLES_EXT = constant.Constant( 'GL_MAX_SAMPLES_EXT', 0x8D57 )
glget.addGLGetConstant( GL_MAX_SAMPLES_EXT, (1,) )
glRenderbufferStorageMultisampleEXT = platform.createExtensionFunction( 
'glRenderbufferStorageMultisampleEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLsizei,constants.GLenum,constants.GLsizei,constants.GLsizei,),
doc='glRenderbufferStorageMultisampleEXT(GLenum(target), GLsizei(samples), GLenum(internalformat), GLsizei(width), GLsizei(height)) -> None',
argNames=('target','samples','internalformat','width','height',),
deprecated=_DEPRECATED,
)


def glInitFramebufferMultisampleEXT():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



"""Convenience module providing common shader entry points

The point of this module is to allow client code to use
OpenGL 2.x style names to reference shader-related operations
even if the local hardware only supports ARB extension-based 
shader rendering.

There are also two utility methods compileProgram and compileShader
which make it easy to create demos which are shader-using.
"""
import logging 
logging.basicConfig()
log = logging.getLogger( 'OpenGL.GL.shaders' )
from OpenGL import GL
from OpenGL.GL.ARB import shader_objects, fragment_shader, vertex_shader, vertex_program
from OpenGL.extensions import alternate

__all__ = [
    'glAttachShader',
    'glDeleteShader',
    'glGetProgramInfoLog',
    'glGetShaderInfoLog',
    'glGetProgramiv',
    'glGetShaderiv',
    'compileProgram',
    'compileShader',
    'GL_VALIDATE_STATUS',
    'GL_LINK_STATUS',
    # automatically added stuff here...
]

def _alt( base, name ):
    if hasattr( GL, base ):
        root = getattr( GL, base )
        if hasattr(root,'__call__'):
            globals()[base] = alternate( 
                getattr(GL,base),
                getattr(module,name)
            )
            __all__.append( base )
        else:
            globals()[base] = root
            __all__.append( base )
        return True 
    return False
_excludes = ['glGetProgramiv']
for module in shader_objects,fragment_shader,vertex_shader,vertex_program:
    for name in dir(module):
        found = None
        for suffix in ('ObjectARB','_ARB','ARB'):
            if name.endswith( suffix ):
                found = False 
                base = name[:-(len(suffix))]
                if base not in _excludes:
                    if _alt( base, name ):
                        found = True
                        break 
        if found is False:
            log.debug( '''Found no alternate for: %s.%s''',
                module.__name__,name,
            )

glAttachShader = alternate( GL.glAttachShader,shader_objects.glAttachObjectARB )
glDetachShader = alternate( GL.glDetachShader,shader_objects.glDetachObjectARB )
glDeleteShader = alternate( GL.glDeleteShader,shader_objects.glDeleteObjectARB )
glGetAttachedShaders = alternate( GL.glGetAttachedShaders, shader_objects.glGetAttachedObjectsARB )

glGetProgramInfoLog = alternate( GL.glGetProgramInfoLog, shader_objects.glGetInfoLogARB )
glGetShaderInfoLog = alternate( GL.glGetShaderInfoLog, shader_objects.glGetInfoLogARB )

glGetShaderiv = alternate( GL.glGetShaderiv, shader_objects.glGetObjectParameterivARB )
glGetProgramiv = alternate( GL.glGetProgramiv, shader_objects.glGetObjectParameterivARB )

GL_VALIDATE_STATUS = GL.GL_VALIDATE_STATUS
GL_COMPILE_STATUS = GL.GL_COMPILE_STATUS
GL_LINK_STATUS = GL.GL_LINK_STATUS
GL_FALSE = GL.GL_FALSE

class ShaderProgram( int ):
    """Integer sub-class with context-manager operation"""
    def __enter__( self ):
        """Start use of the program"""
        glUseProgram( self )
    def __exit__( self, typ, val, tb ):
        """Stop use of the program"""
        glUseProgram( 0 )

def compileProgram(*shaders):
    """Create a new program, attach shaders and validate
    
    shaders -- arbitrary number of shaders to attach to the 
        generated program.
    
    This convenience function is *not* standard OpenGL,
    but it does wind up being fairly useful for demos 
    and the like.  You may wish to copy it to your code 
    base to guard against PyOpenGL changes.
    
    Usage:
    
        shader = compileProgram( 
            compileShader( source, GL_VERTEX_SHADER ),
            compileShader( source2, GL_FRAGMENT_SHADER ),
        )
        glUseProgram( shader )
    
    Note:
        If (and only if) validation of the linked program 
        *passes* then the passed-in shader objects will be 
        deleted from the GL.
    
    returns GLuint shader program reference
    raises RuntimeError when a link/validation failure occurs
    """
    program = glCreateProgram()
    for shader in shaders:
        glAttachShader(program, shader)
    glLinkProgram(program)
    # Validation has to occur *after* linking
    glValidateProgram( program )
    validation = glGetProgramiv( program, GL_VALIDATE_STATUS )
    if validation == GL_FALSE:
        raise RuntimeError(
            """Validation failure (%s): %s"""%(
            validation,
            glGetProgramInfoLog( program ),
        ))
    link_status = glGetProgramiv( program, GL_LINK_STATUS )
    if link_status == GL_FALSE:
        raise RuntimeError(
            """Link failure (%s): %s"""%(
            link_status,
            glGetProgramInfoLog( program ),
        ))
    for shader in shaders:
        glDeleteShader(shader)
    return ShaderProgram( program )
def compileShader( source, shaderType ):
    """Compile shader source of given type
    
    source -- GLSL source-code for the shader
    shaderType -- GLenum GL_VERTEX_SHADER, GL_FRAGMENT_SHADER, etc,
    
    returns GLuint compiled shader reference
    raises RuntimeError when a compilation failure occurs
    """
    if isinstance( source, (str,unicode)):
        source = [ source ]
    shader = glCreateShader(shaderType)
    glShaderSource( shader, source )
    glCompileShader( shader )
    result = glGetShaderiv( shader, GL_COMPILE_STATUS )
    if not(result):
        # TODO: this will be wrong if the user has 
        # disabled traditional unpacking array support.
        raise RuntimeError(
            """Shader compile failure (%s): %s"""%(
                result,
                glGetShaderInfoLog( shader ),
            ),
            source,
            shaderType,
        )
    return shader



'''OpenGL extension SGIX.sprite

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_SGIX_sprite'
_DEPRECATED = False
GL_SPRITE_SGIX = constant.Constant( 'GL_SPRITE_SGIX', 0x8148 )
GL_SPRITE_MODE_SGIX = constant.Constant( 'GL_SPRITE_MODE_SGIX', 0x8149 )
GL_SPRITE_AXIS_SGIX = constant.Constant( 'GL_SPRITE_AXIS_SGIX', 0x814A )
GL_SPRITE_TRANSLATION_SGIX = constant.Constant( 'GL_SPRITE_TRANSLATION_SGIX', 0x814B )
GL_SPRITE_AXIAL_SGIX = constant.Constant( 'GL_SPRITE_AXIAL_SGIX', 0x814C )
GL_SPRITE_OBJECT_ALIGNED_SGIX = constant.Constant( 'GL_SPRITE_OBJECT_ALIGNED_SGIX', 0x814D )
GL_SPRITE_EYE_ALIGNED_SGIX = constant.Constant( 'GL_SPRITE_EYE_ALIGNED_SGIX', 0x814E )
glSpriteParameterfSGIX = platform.createExtensionFunction( 
'glSpriteParameterfSGIX',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLfloat,),
doc='glSpriteParameterfSGIX(GLenum(pname), GLfloat(param)) -> None',
argNames=('pname','param',),
deprecated=_DEPRECATED,
)

glSpriteParameterfvSGIX = platform.createExtensionFunction( 
'glSpriteParameterfvSGIX',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,arrays.GLfloatArray,),
doc='glSpriteParameterfvSGIX(GLenum(pname), GLfloatArray(params)) -> None',
argNames=('pname','params',),
deprecated=_DEPRECATED,
)

glSpriteParameteriSGIX = platform.createExtensionFunction( 
'glSpriteParameteriSGIX',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLint,),
doc='glSpriteParameteriSGIX(GLenum(pname), GLint(param)) -> None',
argNames=('pname','param',),
deprecated=_DEPRECATED,
)

glSpriteParameterivSGIX = platform.createExtensionFunction( 
'glSpriteParameterivSGIX',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,arrays.GLintArray,),
doc='glSpriteParameterivSGIX(GLenum(pname), GLintArray(params)) -> None',
argNames=('pname','params',),
deprecated=_DEPRECATED,
)


def glInitSpriteSGIX():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension EXT.draw_instanced

This module customises the behaviour of the 
OpenGL.raw.GL.EXT.draw_instanced to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension provides the means to render multiple instances of
	an object with a single draw call, and an "instance ID" variable
	which can be used by the vertex program to compute per-instance
	values, typically an object's transform.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/draw_instanced.txt
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions, wrapper
from OpenGL.GL import glget
import ctypes
from OpenGL.raw.GL.EXT.draw_instanced import *
### END AUTOGENERATED SECTION


#! /usr/bin/env python
"""OpenGL-ctypes setup script (setuptools-based)
"""
from distutils.core import setup
import sys, os
sys.path.insert(0, '.' )
import metadata

def is_package( path ):
    return os.path.isfile( os.path.join( path, '__init__.py' ))
def find_packages( root ):
    """Find all packages under this directory"""
    for path, directories, files in os.walk( root ):
        if is_package( path ):
            yield path.replace( '/','.' )

requirements = []
if sys.hexversion < 0x2050000:
    requirements.append( 'ctypes' )

from distutils.command.install_data import install_data
class smart_install_data(install_data):
    def run(self):
        #need to change self.install_dir to the library dir
        install_cmd = self.get_finalized_command('install')
        self.install_dir = getattr(install_cmd, 'install_lib')
        # should create the directory if it doesn't exist!!!
        return install_data.run(self)

if sys.platform == 'win32':
    # binary versions of GLUT and GLE for Win32 (sigh)
    DLL_DIRECTORY = os.path.join('OpenGL','DLLS')
    datafiles = [
        (
            DLL_DIRECTORY, [
                os.path.join( DLL_DIRECTORY,file)
                for file in os.listdir( DLL_DIRECTORY )
            ]
        ),
    ]
else:
    datafiles = []


if __name__ == "__main__":
    setup(
        name = "PyOpenGL",
        packages = list( find_packages('OpenGL') ),
        description = 'Standard OpenGL bindings for Python',
        options = {
            'sdist': {
                'formats': ['gztar','zip'],
                'force_manifest': True,
            },
        },
        data_files = datafiles,
        cmdclass = {'install_data':smart_install_data},
        **metadata.metadata
    )



'''OpenGL extension NV.fragment_program2

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_NV_fragment_program2'
_DEPRECATED = False
GL_MAX_PROGRAM_EXEC_INSTRUCTIONS_NV = constant.Constant( 'GL_MAX_PROGRAM_EXEC_INSTRUCTIONS_NV', 0x88F4 )
GL_MAX_PROGRAM_CALL_DEPTH_NV = constant.Constant( 'GL_MAX_PROGRAM_CALL_DEPTH_NV', 0x88F5 )
GL_MAX_PROGRAM_IF_DEPTH_NV = constant.Constant( 'GL_MAX_PROGRAM_IF_DEPTH_NV', 0x88F6 )
GL_MAX_PROGRAM_LOOP_DEPTH_NV = constant.Constant( 'GL_MAX_PROGRAM_LOOP_DEPTH_NV', 0x88F7 )
GL_MAX_PROGRAM_LOOP_COUNT_NV = constant.Constant( 'GL_MAX_PROGRAM_LOOP_COUNT_NV', 0x88F8 )


def glInitFragmentProgram2NV():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension SGIX.reference_plane

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_SGIX_reference_plane'
_DEPRECATED = False
GL_REFERENCE_PLANE_SGIX = constant.Constant( 'GL_REFERENCE_PLANE_SGIX', 0x817D )
GL_REFERENCE_PLANE_EQUATION_SGIX = constant.Constant( 'GL_REFERENCE_PLANE_EQUATION_SGIX', 0x817E )
glReferencePlaneSGIX = platform.createExtensionFunction( 
'glReferencePlaneSGIX',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(arrays.GLdoubleArray,),
doc='glReferencePlaneSGIX(GLdoubleArray(equation)) -> None',
argNames=('equation',),
deprecated=_DEPRECATED,
)


def glInitReferencePlaneSGIX():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension EXT.separate_shader_objects

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_separate_shader_objects'
_DEPRECATED = False
GL_ACTIVE_PROGRAM_EXT = constant.Constant( 'GL_ACTIVE_PROGRAM_EXT', 0x8B8D )
glUseShaderProgramEXT = platform.createExtensionFunction( 
'glUseShaderProgramEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLuint,),
doc='glUseShaderProgramEXT(GLenum(type), GLuint(program)) -> None',
argNames=('type','program',),
deprecated=_DEPRECATED,
)

glActiveProgramEXT = platform.createExtensionFunction( 
'glActiveProgramEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,),
doc='glActiveProgramEXT(GLuint(program)) -> None',
argNames=('program',),
deprecated=_DEPRECATED,
)

glCreateShaderProgramEXT = platform.createExtensionFunction( 
'glCreateShaderProgramEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=constants.GLuint, 
argTypes=(constants.GLenum,arrays.GLcharArray,),
doc='glCreateShaderProgramEXT(GLenum(type), GLcharArray(string)) -> constants.GLuint',
argNames=('type','string',),
deprecated=_DEPRECATED,
)


def glInitSeparateShaderObjectsEXT():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension ATI.texture_env_combine3

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_ATI_texture_env_combine3'
_DEPRECATED = False
GL_MODULATE_ADD_ATI = constant.Constant( 'GL_MODULATE_ADD_ATI', 0x8744 )
GL_MODULATE_SIGNED_ADD_ATI = constant.Constant( 'GL_MODULATE_SIGNED_ADD_ATI', 0x8745 )
GL_MODULATE_SUBTRACT_ATI = constant.Constant( 'GL_MODULATE_SUBTRACT_ATI', 0x8746 )


def glInitTextureEnvCombine3ATI():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension SGI.depth_pass_instrument

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_SGI_depth_pass_instrument'
_DEPRECATED = False
GL_DEPTH_PASS_INSTRUMENT_SGIX = constant.Constant( 'GL_DEPTH_PASS_INSTRUMENT_SGIX', 0x8310 )
GL_DEPTH_PASS_INSTRUMENT_COUNTERS_SGIX = constant.Constant( 'GL_DEPTH_PASS_INSTRUMENT_COUNTERS_SGIX', 0x8311 )
GL_DEPTH_PASS_INSTRUMENT_MAX_SGIX = constant.Constant( 'GL_DEPTH_PASS_INSTRUMENT_MAX_SGIX', 0x8312 )


def glInitDepthPassInstrumentSGI():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



"""Image-handling routines

### Unresolved:

    Following methods are not yet resolved due to my not being sure how the 
    function should be wrapped:
    
        glCompressedTexImage3D
        glCompressedTexImage2D
        glCompressedTexImage1D
        glCompressedTexSubImage3D
        glCompressedTexSubImage2D
        glCompressedTexSubImage1D
"""
from OpenGL.raw import GL as simple
from OpenGL import images, arrays, wrapper, platform
import ctypes

def asInt( value ):
    if isinstance( value, float ):
        return int(round(value,0))
    return value

## update the image tables with standard image types...
#images.FORMAT_BITS.update( {
#	simple.GL_BITMAP : 1, # must be GL_UNSIGNED_BYTE
#	
#	simple.GL_RED : 8,
#	simple.GL_GREEN : 8,
#	simple.GL_BLUE : 8,
#	simple.GL_ALPHA : 8,
#	simple.GL_LUMINANCE : 8,
#	simple.GL_LUMINANCE_ALPHA : 8,
#	simple.GL_COLOR_INDEX : 8,
#	simple.GL_STENCIL_INDEX : 8,
#	simple.GL_DEPTH_COMPONENT : 8,
#	simple.GL_RGB : 24,
#	simple.GL_BGR : 24,
#
#	simple.GL_RGBA : 32,
#	simple.GL_BGRA : 32,
#	simple.GL_ABGR_EXT : 32,
#	simple.GL_CMYK_EXT : 32,
#
#	simple.GL_CMYKA_EXT : 40,
#	
#	simple.GL_YCRCB_422_SGIX : 8, # must be GL_UNSIGNED_BYTE
#	simple.GL_YCRCB_444_SGIX : 8, # must be GL_UNSIGNED_SHORT
#
#	simple.GL_FORMAT_SUBSAMPLE_24_24_OML : 32, # must be GL_UNSIGNED_INT_10_10_10_2
#	simple.GL_FORMAT_SUBSAMPLE_244_244_OML : 32, # must be GL_UNSIGNED_INT_10_10_10_2
#} )
images.COMPONENT_COUNTS.update( {
    simple.GL_BITMAP : 1, # must be GL_UNSIGNED_BYTE
    
    simple.GL_RED : 1,
    simple.GL_GREEN : 1,
    simple.GL_BLUE : 1,
    simple.GL_ALPHA : 1,
    simple.GL_LUMINANCE : 1,
    simple.GL_LUMINANCE_ALPHA : 2,
    simple.GL_COLOR_INDEX : 1,
    simple.GL_STENCIL_INDEX : 1,
    simple.GL_DEPTH_COMPONENT : 1,
    simple.GL_RGB : 3,
    simple.GL_BGR : 3,

    simple.GL_RGBA : 4,
    simple.GL_BGRA : 4,
    simple.GL_ABGR_EXT : 4,
    simple.GL_CMYK_EXT : 4,

    simple.GL_CMYKA_EXT : 5,
    
    simple.GL_YCRCB_422_SGIX : 1, # must be GL_UNSIGNED_BYTE
    simple.GL_YCRCB_444_SGIX : 1, # must be GL_UNSIGNED_SHORT

    simple.GL_FORMAT_SUBSAMPLE_24_24_OML : 1, # must be GL_UNSIGNED_INT_10_10_10_2
    simple.GL_FORMAT_SUBSAMPLE_244_244_OML : 1, # must be GL_UNSIGNED_INT_10_10_10_2
} )

#images.TYPE_TO_BITS.update( {
#	simple.GL_UNSIGNED_BYTE_3_3_2 : 8,
#	simple.GL_UNSIGNED_BYTE_2_3_3_REV : 8,
#	simple.GL_UNSIGNED_SHORT_4_4_4_4 : 16,
#	simple.GL_UNSIGNED_SHORT_4_4_4_4_REV : 16,
#	simple.GL_UNSIGNED_SHORT_5_5_5_1 : 16,
#	simple.GL_UNSIGNED_SHORT_1_5_5_5_REV : 16,
#	simple.GL_UNSIGNED_SHORT_5_6_5 : 16,
#	simple.GL_UNSIGNED_SHORT_5_6_5_REV : 16,
#	simple.GL_UNSIGNED_INT_8_8_8_8 : 32,
#	simple.GL_UNSIGNED_INT_8_8_8_8_REV : 32,
#	simple.GL_UNSIGNED_INT_10_10_10_2 : 32,
#	simple.GL_UNSIGNED_INT_2_10_10_10_REV : 32,
#	simple.GL_UNSIGNED_BYTE : ctypes.sizeof(simple.GLubyte) * 8,
#	simple.GL_BYTE: ctypes.sizeof(simple.GLbyte) * 8,
#	simple.GL_UNSIGNED_SHORT :  ctypes.sizeof(simple.GLushort) * 8,
#	simple.GL_SHORT :  ctypes.sizeof(simple.GLshort) * 8,
#	simple.GL_UNSIGNED_INT : ctypes.sizeof(simple.GLuint) * 8,
#	simple.GL_INT : ctypes.sizeof(simple.GLint) * 8,
#	simple.GL_FLOAT : ctypes.sizeof(simple.GLfloat) * 8,
#	simple.GL_DOUBLE : ctypes.sizeof(simple.GLdouble) * 8,
#} )
images.TYPE_TO_ARRAYTYPE.update( {
    simple.GL_UNSIGNED_BYTE_3_3_2 : simple.GL_UNSIGNED_BYTE,
    simple.GL_UNSIGNED_BYTE_2_3_3_REV : simple.GL_UNSIGNED_BYTE,
    simple.GL_UNSIGNED_SHORT_4_4_4_4 : simple.GL_UNSIGNED_SHORT,
    simple.GL_UNSIGNED_SHORT_4_4_4_4_REV : simple.GL_UNSIGNED_SHORT,
    simple.GL_UNSIGNED_SHORT_5_5_5_1 : simple.GL_UNSIGNED_SHORT,
    simple.GL_UNSIGNED_SHORT_1_5_5_5_REV : simple.GL_UNSIGNED_SHORT,
    simple.GL_UNSIGNED_SHORT_5_6_5 : simple.GL_UNSIGNED_SHORT,
    simple.GL_UNSIGNED_SHORT_5_6_5_REV : simple.GL_UNSIGNED_SHORT,
    simple.GL_UNSIGNED_INT_8_8_8_8 : simple.GL_UNSIGNED_INT,
    simple.GL_UNSIGNED_INT_8_8_8_8_REV : simple.GL_UNSIGNED_INT,
    simple.GL_UNSIGNED_INT_10_10_10_2 : simple.GL_UNSIGNED_INT,
    simple.GL_UNSIGNED_INT_2_10_10_10_REV : simple.GL_UNSIGNED_INT,
    simple.GL_UNSIGNED_BYTE : simple.GL_UNSIGNED_BYTE,
    simple.GL_BYTE: simple.GL_BYTE,
    simple.GL_UNSIGNED_SHORT : simple.GL_UNSIGNED_SHORT,
    simple.GL_SHORT :  simple.GL_SHORT,
    simple.GL_UNSIGNED_INT : simple.GL_UNSIGNED_INT,
    simple.GL_INT : simple.GL_INT,
    simple.GL_FLOAT : simple.GL_FLOAT,
    simple.GL_DOUBLE : simple.GL_DOUBLE,
    simple.GL_BITMAP : simple.GL_UNSIGNED_BYTE,
} )
images.TIGHT_PACK_FORMATS.update({
    simple.GL_UNSIGNED_BYTE_3_3_2 : 3,
    simple.GL_UNSIGNED_BYTE_2_3_3_REV : 3,
    simple.GL_UNSIGNED_SHORT_4_4_4_4 : 4,
    simple.GL_UNSIGNED_SHORT_4_4_4_4_REV : 4,
    simple.GL_UNSIGNED_SHORT_5_5_5_1 : 4,
    simple.GL_UNSIGNED_SHORT_1_5_5_5_REV : 4,
    simple.GL_UNSIGNED_SHORT_5_6_5 : 3,
    simple.GL_UNSIGNED_SHORT_5_6_5_REV : 3,
    simple.GL_UNSIGNED_INT_8_8_8_8 : 4,
    simple.GL_UNSIGNED_INT_8_8_8_8_REV : 4,
    simple.GL_UNSIGNED_INT_10_10_10_2 : 4,
    simple.GL_UNSIGNED_INT_2_10_10_10_REV : 4,
    simple.GL_BITMAP: 8, # single bits, 8 of them...
})

images.RANK_PACKINGS.update( {
    4: [
        (simple.glPixelStorei,simple.GL_PACK_SKIP_VOLUMES_SGIS, 0),
        (simple.glPixelStorei,simple.GL_PACK_IMAGE_DEPTH_SGIS, 0),
        (simple.glPixelStorei,simple.GL_PACK_ALIGNMENT, 1),
    ],
    3: [
        (simple.glPixelStorei,simple.GL_PACK_SKIP_IMAGES, 0),
        (simple.glPixelStorei,simple.GL_PACK_IMAGE_HEIGHT, 0),
        (simple.glPixelStorei,simple.GL_PACK_ALIGNMENT, 1),
    ],
    2: [
        (simple.glPixelStorei,simple.GL_PACK_ROW_LENGTH, 0),
        (simple.glPixelStorei,simple.GL_PACK_SKIP_ROWS, 0),
        (simple.glPixelStorei,simple.GL_PACK_ALIGNMENT, 1),
    ],
    1: [
        (simple.glPixelStorei,simple.GL_PACK_SKIP_PIXELS, 0),
        (simple.glPixelStorei,simple.GL_PACK_ALIGNMENT, 1),
    ],
} )


__all__ = (
    'glReadPixels',
    'glReadPixelsb',
    'glReadPixelsd',
    'glReadPixelsf',
    'glReadPixelsi',
    'glReadPixelss',
    'glReadPixelsub',
    'glReadPixelsui',
    'glReadPixelsus',
    
    'glGetTexImage',
    
    'glDrawPixels',
    'glDrawPixelsb',
    'glDrawPixelsf',
    'glDrawPixelsi',
    'glDrawPixelss',
    'glDrawPixelsub',
    'glDrawPixelsui',
    'glDrawPixelsus',
    
    
    'glTexSubImage2D',
    'glTexSubImage1D',
    #'glTexSubImage3D',
    
    'glTexImage1D',
    'glTexImage2D',
    #'glTexImage3D',
    
    'glGetTexImageb',
    'glGetTexImaged',
    'glGetTexImagef',
    'glGetTexImagei',
    'glGetTexImages',
    'glGetTexImageub',
    'glGetTexImageui',
    'glGetTexImageus',
    'glTexImage1Db',
    'glTexImage2Db',
    #'glTexImage3Db',
    'glTexSubImage1Db',
    'glTexSubImage2Db',
    #'glTexSubImage3Db',
    'glTexImage1Df',
    'glTexImage2Df',
    #'glTexImage3Df',
    'glTexSubImage1Df',
    'glTexSubImage2Df',
    #'glTexSubImage3Df',
    'glTexImage1Di',
    'glTexImage2Di',
    #'glTexImage3Di',
    'glTexSubImage1Di',
    'glTexSubImage2Di',
    #'glTexSubImage3Di',
    'glTexImage1Ds',
    'glTexImage2Ds',
    #'glTexImage3Ds',
    'glTexSubImage1Ds',
    'glTexSubImage2Ds',
    #'glTexSubImage3Ds',
    'glTexImage1Dub',
    'glTexImage2Dub',
    #'glTexImage3Dub',
    'glTexSubImage1Dub',
    'glTexSubImage2Dub',
    #'glTexSubImage3Dub',
    'glTexImage1Dui',
    'glTexImage2Dui',
    #'glTexImage3Dui',
    'glTexSubImage1Dui',
    'glTexSubImage2Dui',
    #'glTexSubImage3Dui',
    'glTexImage1Dus',
    'glTexImage2Dus',
    #'glTexImage3Dus',
    'glTexSubImage1Dus',
    'glTexSubImage2Dus',
    #'glTexSubImage3Dus',
    
    #'glColorTable',
    #'glGetColorTable',
    #'glColorSubTable',
    
    #'glConvolutionFilter1D',
    #'glConvolutionFilter2D',
    #'glGetConvolutionFilter',
    #'glSeparableFilter2D',
    #'glGetSeparableFilter',
    
    #'glGetMinmax',
)

for suffix,type in [
    ('b',simple.GL_BYTE),
    ('d',simple.GL_DOUBLE),
    ('f',simple.GL_FLOAT),
    ('i',simple.GL_INT),
    ('s',simple.GL_SHORT),
    ('ub',simple.GL_UNSIGNED_BYTE),
    ('ui',simple.GL_UNSIGNED_INT),
    ('us',simple.GL_UNSIGNED_SHORT),
]:
    def glReadPixels( x,y,width,height,format,type=type, array=None ):
        """Read specified pixels from the current display buffer
        
        This typed version returns data in your specified default 
        array data-type format, or in the passed array, which will 
        be converted to the array-type required by the format.
        """
        x,y,width,height = asInt(x),asInt(y),asInt(width),asInt(height)
        arrayType = arrays.GL_CONSTANT_TO_ARRAY_TYPE[ images.TYPE_TO_ARRAYTYPE.get(type,type) ]
        if array is None:
            array = images.SetupPixelRead( format, (width,height), type )
        else:
            array = arrayType.asArray( array )
        imageData = arrayType.voidDataPointer( array )
        simple.glReadPixels( 
            x,y,
            width, height,
            format,type, 
            imageData
        )
        return array
    globals()["glReadPixels%s"%(suffix,)] = glReadPixels
    def glGetTexImage( target, level,format,type=type ):
        """Get a texture-level as an image"""
        from OpenGL.GL import glget
        dims = [glget.glGetTexLevelParameteriv( target, level, simple.GL_TEXTURE_WIDTH )]
        if target != simple.GL_TEXTURE_1D:
            dims.append( glget.glGetTexLevelParameteriv( target, level, simple.GL_TEXTURE_HEIGHT ) )
            if target != simple.GL_TEXTURE_2D:
                dims.append( glget.glGetTexLevelParameteriv( target, level, simple.GL_TEXTURE_DEPTH ) )
        array = images.SetupPixelRead( format, tuple(dims), type )
        arrayType = arrays.GL_CONSTANT_TO_ARRAY_TYPE[ images.TYPE_TO_ARRAYTYPE.get(type,type) ]
        simple.glGetTexImage( 
            target, level, format, type, ctypes.c_void_p( arrayType.dataPointer(array)) 
        )
        return array
    globals()["glGetTexImage%s"%(suffix,)] = glGetTexImage
##	def glGetTexSubImage( target, level,format,type ):
##		"""Get a texture-level as an image"""
##		from OpenGL.GL import glget
##		dims = [glget.glGetTexLevelParameteriv( target, level, simple.GL_TEXTURE_WIDTH )]
##		if target != simple.GL_TEXTURE_1D:
##			dims.append( glget.glGetTexLevelParameteriv( target, level, simple.GL_TEXTURE_HEIGHT ) )
##			if target != simple.GL_TEXTURE_2D:
##				dims.append( glget.glGetTexLevelParameteriv( target, level, simple.GL_TEXTURE_DEPTH ) )
##		array = images.SetupPixelRead( format, tuple(dims), type )
##		arrayType = arrays.GL_CONSTANT_TO_ARRAY_TYPE[ images.TYPE_TO_ARRAYTYPE.get(type,type) ]
##		simple.glGetTexImage( 
##			target, level, format, type, ctypes.c_void_p( arrayType.dataPointer(array)) 
##		)
##		return array
##	"%s = glGetTexImage"%(suffix)
    del suffix,type
# Now the real glReadPixels...
def glReadPixels( x,y,width,height,format,type, array=None, outputType=str ):
    """Read specified pixels from the current display buffer
    
    x,y,width,height -- location and dimensions of the image to read 
        from the buffer
    format -- pixel format for the resulting data
    type -- data-format for the resulting data
    array -- optional array/offset into which to store the value
    outputType -- default (str) provides string output of the 
        results iff OpenGL.UNSIGNED_BYTE_IMAGES_AS_STRING is True 
        and type == GL_UNSIGNED_BYTE.  Any other value will cause 
        output in the default array output format.
    
    returns the pixel data array in the format defined by the 
    format, type and outputType
    """
    x,y,width,height = asInt(x),asInt(y),asInt(width),asInt(height)
    
    arrayType = arrays.GL_CONSTANT_TO_ARRAY_TYPE[ images.TYPE_TO_ARRAYTYPE.get(type,type) ]
    if array is None:
        array = images.SetupPixelRead( format, (width,height), type )
    else:
        array = arrayType.asArray( array )
    imageData = arrayType.voidDataPointer( array )
    simple.glReadPixels( 
        x,y,width,height,
        format,type, 
        imageData
    )
    if outputType is str:
        return images.returnFormat( array, type )
    else:
        return array

def glGetTexImage( target, level,format,type, outputType=str ):
    """Get a texture-level as an image
    
    target -- enum constant for the texture engine to be read 
    level -- the mip-map level to read 
    format -- image format to read out the data 
    type -- data-type into which to read the data
    
    outputType -- default (str) provides string output of the 
        results iff OpenGL.UNSIGNED_BYTE_IMAGES_AS_STRING is True 
        and type == GL_UNSIGNED_BYTE.  Any other value will cause 
        output in the default array output format.
    
    returns the pixel data array in the format defined by the 
    format, type and outputType
    """
    from OpenGL.GL import glget
    dims = [glget.glGetTexLevelParameteriv( target, level, simple.GL_TEXTURE_WIDTH )]
    if target != simple.GL_TEXTURE_1D:
        dims.append( glget.glGetTexLevelParameteriv( target, level, simple.GL_TEXTURE_HEIGHT ) )
        if target != simple.GL_TEXTURE_2D:
            dims.append( glget.glGetTexLevelParameteriv( target, level, simple.GL_TEXTURE_DEPTH ) )
    array = images.SetupPixelRead( format, tuple(dims), type )
    arrayType = arrays.GL_CONSTANT_TO_ARRAY_TYPE[ images.TYPE_TO_ARRAYTYPE.get(type,type) ]
    simple.glGetTexImage( 
        target, level, format, type, ctypes.c_void_p( arrayType.dataPointer(array)) 
    )
    if outputType is str:
        return images.returnFormat( array, type )
    else:
        return array


INT_DIMENSION_NAMES = [
    'width','height','depth','x','y','z',
    'xoffset','yoffset','zoffset',
    'start', 'count',
]
def asWrapper( value ):
    if not isinstance( value, wrapper.Wrapper ):
        return wrapper.wrapper( value )
    return value

def asIntConverter( value, *args ):
    if isinstance( value, float ):
        return int(round(value,0))
    return value

def setDimensionsAsInts( baseOperation ):
    """Set arguments with names in INT_DIMENSION_NAMES to asInt processing"""
    baseOperation = asWrapper( baseOperation )
    argNames = getattr( baseOperation, 'pyConverterNames', baseOperation.argNames )
    for i,argName in enumerate(argNames):
        if argName in INT_DIMENSION_NAMES:
            baseOperation.setPyConverter( argName, asIntConverter )
    return baseOperation

    

class ImageInputConverter( object ):
    def __init__( self, rank, pixelsName=None, typeName='type' ):
        self.rank = rank
        self.typeName = typeName
        self.pixelsName = pixelsName
    def finalise( self, wrapper ):
        """Get our pixel index from the wrapper"""
        self.typeIndex = wrapper.pyArgIndex( self.typeName )
        self.pixelsIndex = wrapper.pyArgIndex( self.pixelsName )
    def __call__( self, arg, baseOperation, pyArgs ):
        """pyConverter for the pixels argument"""
        images.setupDefaultTransferMode()
        images.rankPacking( self.rank )
        type = pyArgs[ self.typeIndex ]
        arrayType = arrays.GL_CONSTANT_TO_ARRAY_TYPE[ images.TYPE_TO_ARRAYTYPE[ type ] ]
        return arrayType.asArray( arg )
#	def cResolver( self, array ):
#		return array
#		return ctypes.c_void_p( arrays.ArrayDatatype.dataPointer( array ) )
    
class TypedImageInputConverter( ImageInputConverter ):
    def __init__( self, rank, pixelsName, arrayType, typeName=None ):
        self.rank = rank
        self.arrayType = arrayType
        self.pixelsName = pixelsName
        self.typeName = typeName
    def __call__( self, arg, baseOperation, pyArgs ):
        """The pyConverter for the pixels"""
        images.setupDefaultTransferMode()
        images.rankPacking( self.rank )
        return self.arrayType.asArray( arg )
    def finalise( self, wrapper ):
        """Get our pixel index from the wrapper"""
        self.pixelsIndex = wrapper.pyArgIndex( self.pixelsName )
    def width( self, pyArgs, index, wrappedOperation ):
        """Extract the width from the pixels argument"""
        return self.arrayType.dimensions( pyArgs[self.pixelsIndex] )[0]
    def height( self, pyArgs, index, wrappedOperation ):
        """Extract the height from the pixels argument"""
        return self.arrayType.dimensions( pyArgs[self.pixelsIndex] )[1]
    def depth( self, pyArgs, index, wrappedOperation ):
        """Extract the depth from the pixels argument"""
        return self.arrayType.dimensions( pyArgs[self.pixelsIndex] )[2]
    def type( self, pyArgs, index, wrappedOperation ):
        """Provide the item-type argument from our stored value
        
        This is used for pre-bound processing where we want to provide 
        the type by implication...
        """
        return self.typeName

class CompressedImageConverter( object ):
    def finalise( self, wrapper ):
        """Get our pixel index from the wrapper"""
        self.dataIndex = wrapper.pyArgIndex( 'data' )
    def __call__( self, pyArgs, index, wrappedOperation ):
        """Create a data-size measurement for our image"""
        arg = pyArgs[ self.dataIndex ]
        return arrays.ArrayType.arrayByteCount( arg )



DIMENSION_NAMES = (
    'width','height','depth'
)
PIXEL_NAMES = (
    'pixels', 'row', 'column',
)
DATA_SIZE_NAMES = (
    'imageSize',
)

def setImageInput( 
    baseOperation, arrayType=None, dimNames=DIMENSION_NAMES, 
    pixelName="pixels", typeName=None 
):
    """Determine how to convert "pixels" into an image-compatible argument"""
    baseOperation = asWrapper( baseOperation )
    # rank is the count of width,height,depth arguments...
    rank = len([
        # rank is the number of dims we want, not the number we give...
        argName for argName in baseOperation.argNames 
        if argName in dimNames
    ]) + 1
    if arrayType:
        converter = TypedImageInputConverter( rank, pixelName, arrayType, typeName=typeName )
        for i,argName in enumerate(baseOperation.argNames):
            if argName in dimNames:
                baseOperation.setPyConverter( argName )
                baseOperation.setCConverter( argName, getattr(converter,argName) )
            elif argName == 'type' and typeName is not None:
                baseOperation.setPyConverter( argName )
                baseOperation.setCConverter( argName, converter.type )
    else:
        converter = ImageInputConverter( rank, pixelsName=pixelName, typeName=typeName or 'type' )
    for argName in baseOperation.argNames:
        if argName in DATA_SIZE_NAMES:
            baseOperation.setPyConverter( argName )
            baseOperation.setCConverter( argName, converter.imageDataSize )
    baseOperation.setPyConverter(
        pixelName, converter,
    )
#	baseOperation.setCResolver(
#		pixelName, converter.cResolver
#	)
    return baseOperation

glDrawPixels = setDimensionsAsInts(
    setImageInput(
        simple.glDrawPixels
    )
)
glTexSubImage2D = setDimensionsAsInts(
    setImageInput(
        simple.glTexSubImage2D
    )
)
glTexSubImage1D = setDimensionsAsInts(
    setImageInput(
        simple.glTexSubImage1D
    )
)
glTexImage2D = setDimensionsAsInts(
    setImageInput(
        simple.glTexImage2D
    )
)
glTexImage1D = setDimensionsAsInts(
    setImageInput(
        simple.glTexImage1D
    )
)

def typedImageFunction( suffix, arrayConstant,  baseFunction ):
    """Produce a typed version of the given image function"""
    functionName = baseFunction.__name__
    functionName = '%(functionName)s%(suffix)s'%locals()
    if baseFunction:
        arrayType = arrays.GL_CONSTANT_TO_ARRAY_TYPE[ arrayConstant ]
        function = setDimensionsAsInts(
            setImageInput(
                baseFunction, 
                arrayType,
                typeName = arrayConstant,
            )
        )
        return functionName, function
    else:
        return functionName, baseFunction

def _setDataSize( baseFunction, argument='imageSize' ):
    """Set the data-size value to come from the data field"""
    if baseFunction:
        converter = CompressedImageConverter()
        return asWrapper( baseFunction ).setPyConverter(
            argument
        ).setCConverter( argument, converter )
    else:
        return baseFunction

def compressedImageFunction( baseFunction ):
    """Set the imageSize and dimensions-as-ints converters for baseFunction"""
    if baseFunction:
        return setDimensionsAsInts(
            _setDataSize( 
                baseFunction, argument='imageSize'
            )
        )
    else:
        return baseFunction

for suffix,arrayConstant in [
    ('b', simple.GL_BYTE),
    ('f', simple.GL_FLOAT),
    ('i', simple.GL_INT),
    ('s', simple.GL_SHORT),
    ('ub', simple.GL_UNSIGNED_BYTE),
    ('ui', simple.GL_UNSIGNED_INT),
    ('us', simple.GL_UNSIGNED_SHORT),
]:
    for functionName in (
        'glTexImage1D','glTexImage2D',
        'glTexSubImage1D','glTexSubImage2D',
        'glDrawPixels',
        #'glTexSubImage3D','glTexImage3D', # extension/1.2 standard
    ):
        functionName, function = typedImageFunction(
            suffix, arrayConstant, getattr(simple,functionName),
        )
        globals()[functionName] = function
        del function, functionName
    del suffix,arrayConstant



'''OpenGL extension ARB.texture_buffer_object

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_ARB_texture_buffer_object'
_DEPRECATED = False
GL_TEXTURE_BUFFER_ARB = constant.Constant( 'GL_TEXTURE_BUFFER_ARB', 0x8C2A )
glget.addGLGetConstant( GL_TEXTURE_BUFFER_ARB, (1,) )
GL_MAX_TEXTURE_BUFFER_SIZE_ARB = constant.Constant( 'GL_MAX_TEXTURE_BUFFER_SIZE_ARB', 0x8C2B )
glget.addGLGetConstant( GL_MAX_TEXTURE_BUFFER_SIZE_ARB, (1,) )
GL_TEXTURE_BINDING_BUFFER_ARB = constant.Constant( 'GL_TEXTURE_BINDING_BUFFER_ARB', 0x8C2C )
glget.addGLGetConstant( GL_TEXTURE_BINDING_BUFFER_ARB, (1,) )
GL_TEXTURE_BUFFER_DATA_STORE_BINDING_ARB = constant.Constant( 'GL_TEXTURE_BUFFER_DATA_STORE_BINDING_ARB', 0x8C2D )
glget.addGLGetConstant( GL_TEXTURE_BUFFER_DATA_STORE_BINDING_ARB, (1,) )
GL_TEXTURE_BUFFER_FORMAT_ARB = constant.Constant( 'GL_TEXTURE_BUFFER_FORMAT_ARB', 0x8C2E )
glget.addGLGetConstant( GL_TEXTURE_BUFFER_FORMAT_ARB, (1,) )
glTexBufferARB = platform.createExtensionFunction( 
'glTexBufferARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,constants.GLuint,),
doc='glTexBufferARB(GLenum(target), GLenum(internalformat), GLuint(buffer)) -> None',
argNames=('target','internalformat','buffer',),
deprecated=_DEPRECATED,
)


def glInitTextureBufferObjectARB():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension INGR.color_clamp

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_INGR_color_clamp'
_DEPRECATED = False
GL_RED_MIN_CLAMP_INGR = constant.Constant( 'GL_RED_MIN_CLAMP_INGR', 0x8560 )
glget.addGLGetConstant( GL_RED_MIN_CLAMP_INGR, (1,) )
GL_GREEN_MIN_CLAMP_INGR = constant.Constant( 'GL_GREEN_MIN_CLAMP_INGR', 0x8561 )
glget.addGLGetConstant( GL_GREEN_MIN_CLAMP_INGR, (1,) )
GL_BLUE_MIN_CLAMP_INGR = constant.Constant( 'GL_BLUE_MIN_CLAMP_INGR', 0x8562 )
glget.addGLGetConstant( GL_BLUE_MIN_CLAMP_INGR, (1,) )
GL_ALPHA_MIN_CLAMP_INGR = constant.Constant( 'GL_ALPHA_MIN_CLAMP_INGR', 0x8563 )
glget.addGLGetConstant( GL_ALPHA_MIN_CLAMP_INGR, (1,) )
GL_RED_MAX_CLAMP_INGR = constant.Constant( 'GL_RED_MAX_CLAMP_INGR', 0x8564 )
glget.addGLGetConstant( GL_RED_MAX_CLAMP_INGR, (1,) )
GL_GREEN_MAX_CLAMP_INGR = constant.Constant( 'GL_GREEN_MAX_CLAMP_INGR', 0x8565 )
glget.addGLGetConstant( GL_GREEN_MAX_CLAMP_INGR, (1,) )
GL_BLUE_MAX_CLAMP_INGR = constant.Constant( 'GL_BLUE_MAX_CLAMP_INGR', 0x8566 )
glget.addGLGetConstant( GL_BLUE_MAX_CLAMP_INGR, (1,) )
GL_ALPHA_MAX_CLAMP_INGR = constant.Constant( 'GL_ALPHA_MAX_CLAMP_INGR', 0x8567 )
glget.addGLGetConstant( GL_ALPHA_MAX_CLAMP_INGR, (1,) )


def glInitColorClampINGR():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension SGIX.async

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_SGIX_async'
_DEPRECATED = False
GL_ASYNC_MARKER_SGIX = constant.Constant( 'GL_ASYNC_MARKER_SGIX', 0x8329 )
glget.addGLGetConstant( GL_ASYNC_MARKER_SGIX, (1,) )
glAsyncMarkerSGIX = platform.createExtensionFunction( 
'glAsyncMarkerSGIX',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,),
doc='glAsyncMarkerSGIX(GLuint(marker)) -> None',
argNames=('marker',),
deprecated=_DEPRECATED,
)

glFinishAsyncSGIX = platform.createExtensionFunction( 
'glFinishAsyncSGIX',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=constants.GLint, 
argTypes=(arrays.GLuintArray,),
doc='glFinishAsyncSGIX(GLuintArray(markerp)) -> constants.GLint',
argNames=('markerp',),
deprecated=_DEPRECATED,
)

glPollAsyncSGIX = platform.createExtensionFunction( 
'glPollAsyncSGIX',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=constants.GLint, 
argTypes=(arrays.GLuintArray,),
doc='glPollAsyncSGIX(GLuintArray(markerp)) -> constants.GLint',
argNames=('markerp',),
deprecated=_DEPRECATED,
)

glGenAsyncMarkersSGIX = platform.createExtensionFunction( 
'glGenAsyncMarkersSGIX',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=constants.GLuint, 
argTypes=(constants.GLsizei,),
doc='glGenAsyncMarkersSGIX(GLsizei(range)) -> constants.GLuint',
argNames=('range',),
deprecated=_DEPRECATED,
)

glDeleteAsyncMarkersSGIX = platform.createExtensionFunction( 
'glDeleteAsyncMarkersSGIX',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLsizei,),
doc='glDeleteAsyncMarkersSGIX(GLuint(marker), GLsizei(range)) -> None',
argNames=('marker','range',),
deprecated=_DEPRECATED,
)

glIsAsyncMarkerSGIX = platform.createExtensionFunction( 
'glIsAsyncMarkerSGIX',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=constants.GLboolean, 
argTypes=(constants.GLuint,),
doc='glIsAsyncMarkerSGIX(GLuint(marker)) -> constants.GLboolean',
argNames=('marker',),
deprecated=_DEPRECATED,
)


def glInitAsyncSGIX():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



import ctypes
try:
    from ctypes.util import find_library
except ImportError, err:
    from ctypes.util import findLib 
    def find_library( string ):
        return findLib( string )[0]

GL = OpenGL = ctypes.CDLL( find_library('GL'), mode=ctypes.RTLD_GLOBAL )
GLU = ctypes.CDLL( find_library('GLU'), mode=ctypes.RTLD_GLOBAL )
# glut shouldn't need to be global IIUC
GLUT = ctypes.CDLL( find_library('glut'), mode=ctypes.RTLD_GLOBAL )
print 'GL', GL
print 'GLU', GLU
print 'GLUT', GLUT
print GLUT.glutSolidTeapot


'''OpenGL extension SUN.mesh_array

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_SUN_mesh_array'
_DEPRECATED = False
GL_QUAD_MESH_SUN = constant.Constant( 'GL_QUAD_MESH_SUN', 0x8614 )
GL_TRIANGLE_MESH_SUN = constant.Constant( 'GL_TRIANGLE_MESH_SUN', 0x8615 )
glDrawMeshArraysSUN = platform.createExtensionFunction( 
'glDrawMeshArraysSUN',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLint,constants.GLsizei,constants.GLsizei,),
doc='glDrawMeshArraysSUN(GLenum(mode), GLint(first), GLsizei(count), GLsizei(width)) -> None',
argNames=('mode','first','count','width',),
deprecated=_DEPRECATED,
)


def glInitMeshArraySUN():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension ATI.separate_stencil

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_ATI_separate_stencil'
_DEPRECATED = False
GL_STENCIL_BACK_FUNC_ATI = constant.Constant( 'GL_STENCIL_BACK_FUNC_ATI', 0x8800 )
GL_STENCIL_BACK_FAIL_ATI = constant.Constant( 'GL_STENCIL_BACK_FAIL_ATI', 0x8801 )
GL_STENCIL_BACK_PASS_DEPTH_FAIL_ATI = constant.Constant( 'GL_STENCIL_BACK_PASS_DEPTH_FAIL_ATI', 0x8802 )
GL_STENCIL_BACK_PASS_DEPTH_PASS_ATI = constant.Constant( 'GL_STENCIL_BACK_PASS_DEPTH_PASS_ATI', 0x8803 )
glStencilOpSeparateATI = platform.createExtensionFunction( 
'glStencilOpSeparateATI',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,constants.GLenum,constants.GLenum,),
doc='glStencilOpSeparateATI(GLenum(face), GLenum(sfail), GLenum(dpfail), GLenum(dppass)) -> None',
argNames=('face','sfail','dpfail','dppass',),
deprecated=_DEPRECATED,
)

glStencilFuncSeparateATI = platform.createExtensionFunction( 
'glStencilFuncSeparateATI',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,constants.GLint,constants.GLuint,),
doc='glStencilFuncSeparateATI(GLenum(frontfunc), GLenum(backfunc), GLint(ref), GLuint(mask)) -> None',
argNames=('frontfunc','backfunc','ref','mask',),
deprecated=_DEPRECATED,
)


def glInitSeparateStencilATI():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension SGIS.texture4D

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_SGIS_texture4D'
_DEPRECATED = False
GL_PACK_SKIP_VOLUMES_SGIS = constant.Constant( 'GL_PACK_SKIP_VOLUMES_SGIS', 0x8130 )
GL_PACK_IMAGE_DEPTH_SGIS = constant.Constant( 'GL_PACK_IMAGE_DEPTH_SGIS', 0x8131 )
GL_UNPACK_SKIP_VOLUMES_SGIS = constant.Constant( 'GL_UNPACK_SKIP_VOLUMES_SGIS', 0x8132 )
GL_UNPACK_IMAGE_DEPTH_SGIS = constant.Constant( 'GL_UNPACK_IMAGE_DEPTH_SGIS', 0x8133 )
GL_TEXTURE_4D_SGIS = constant.Constant( 'GL_TEXTURE_4D_SGIS', 0x8134 )
GL_PROXY_TEXTURE_4D_SGIS = constant.Constant( 'GL_PROXY_TEXTURE_4D_SGIS', 0x8135 )
GL_TEXTURE_4DSIZE_SGIS = constant.Constant( 'GL_TEXTURE_4DSIZE_SGIS', 0x8136 )
GL_TEXTURE_WRAP_Q_SGIS = constant.Constant( 'GL_TEXTURE_WRAP_Q_SGIS', 0x8137 )
GL_MAX_4D_TEXTURE_SIZE_SGIS = constant.Constant( 'GL_MAX_4D_TEXTURE_SIZE_SGIS', 0x8138 )
GL_TEXTURE_4D_BINDING_SGIS = constant.Constant( 'GL_TEXTURE_4D_BINDING_SGIS', 0x814F )
glTexImage4DSGIS = platform.createExtensionFunction( 
'glTexImage4DSGIS',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLint,constants.GLenum,constants.GLsizei,constants.GLsizei,constants.GLsizei,constants.GLsizei,constants.GLint,constants.GLenum,constants.GLenum,ctypes.c_void_p,),
doc='glTexImage4DSGIS(GLenum(target), GLint(level), GLenum(internalformat), GLsizei(width), GLsizei(height), GLsizei(depth), GLsizei(size4d), GLint(border), GLenum(format), GLenum(type), c_void_p(pixels)) -> None',
argNames=('target','level','internalformat','width','height','depth','size4d','border','format','type','pixels',),
deprecated=_DEPRECATED,
)

glTexSubImage4DSGIS = platform.createExtensionFunction( 
'glTexSubImage4DSGIS',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLint,constants.GLint,constants.GLint,constants.GLint,constants.GLint,constants.GLsizei,constants.GLsizei,constants.GLsizei,constants.GLsizei,constants.GLenum,constants.GLenum,ctypes.c_void_p,),
doc='glTexSubImage4DSGIS(GLenum(target), GLint(level), GLint(xoffset), GLint(yoffset), GLint(zoffset), GLint(woffset), GLsizei(width), GLsizei(height), GLsizei(depth), GLsizei(size4d), GLenum(format), GLenum(type), c_void_p(pixels)) -> None',
argNames=('target','level','xoffset','yoffset','zoffset','woffset','width','height','depth','size4d','format','type','pixels',),
deprecated=_DEPRECATED,
)


def glInitTexture4DSGIS():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension SGIS.sharpen_texture

This module customises the behaviour of the 
OpenGL.raw.GL.SGIS.sharpen_texture to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension introduces texture magnification filters that sharpen
	the resulting image by extrapolating from the level 1 image to the
	level 0 image.  Sharpening can be enabled for all color channels, for
	the alpha channel only, or for the red, green, and blue channels only.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/SGIS/sharpen_texture.txt
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions, wrapper
from OpenGL.GL import glget
import ctypes
from OpenGL.raw.GL.SGIS.sharpen_texture import *
### END AUTOGENERATED SECTION


'''OpenGL extension EXT.texture_sRGB

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_texture_sRGB'
_DEPRECATED = False
GL_SRGB_EXT = constant.Constant( 'GL_SRGB_EXT', 0x8C40 )
GL_SRGB8_EXT = constant.Constant( 'GL_SRGB8_EXT', 0x8C41 )
GL_SRGB_ALPHA_EXT = constant.Constant( 'GL_SRGB_ALPHA_EXT', 0x8C42 )
GL_SRGB8_ALPHA8_EXT = constant.Constant( 'GL_SRGB8_ALPHA8_EXT', 0x8C43 )
GL_SLUMINANCE_ALPHA_EXT = constant.Constant( 'GL_SLUMINANCE_ALPHA_EXT', 0x8C44 )
GL_SLUMINANCE8_ALPHA8_EXT = constant.Constant( 'GL_SLUMINANCE8_ALPHA8_EXT', 0x8C45 )
GL_SLUMINANCE_EXT = constant.Constant( 'GL_SLUMINANCE_EXT', 0x8C46 )
GL_SLUMINANCE8_EXT = constant.Constant( 'GL_SLUMINANCE8_EXT', 0x8C47 )
GL_COMPRESSED_SRGB_EXT = constant.Constant( 'GL_COMPRESSED_SRGB_EXT', 0x8C48 )
GL_COMPRESSED_SRGB_ALPHA_EXT = constant.Constant( 'GL_COMPRESSED_SRGB_ALPHA_EXT', 0x8C49 )
GL_COMPRESSED_SLUMINANCE_EXT = constant.Constant( 'GL_COMPRESSED_SLUMINANCE_EXT', 0x8C4A )
GL_COMPRESSED_SLUMINANCE_ALPHA_EXT = constant.Constant( 'GL_COMPRESSED_SLUMINANCE_ALPHA_EXT', 0x8C4B )
GL_COMPRESSED_SRGB_S3TC_DXT1_EXT = constant.Constant( 'GL_COMPRESSED_SRGB_S3TC_DXT1_EXT', 0x8C4C )
GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT1_EXT = constant.Constant( 'GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT1_EXT', 0x8C4D )
GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT3_EXT = constant.Constant( 'GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT3_EXT', 0x8C4E )
GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT5_EXT = constant.Constant( 'GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT5_EXT', 0x8C4F )


def glInitTextureSrgbEXT():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



"""VertexBufferObject helper class"""
from OpenGL import GL
from OpenGL.arrays.arraydatatype import ArrayDatatype
from OpenGL.arrays.formathandler import FormatHandler
from OpenGL.GL.ARB import vertex_buffer_object
from OpenGL import constants, error
import ctypes,logging
log = logging.getLogger( 'OpenGL.arrays.vbo' )


import weakref
__all__ = ('VBO','VBOHandler','mapVBO')

class Implementation( object ):
    """Abstraction point for the various implementations that can be used
    """
    available = False
    def _arbname( self, name ):
        return (
            (name.startswith( 'gl' ) and name.endswith( 'ARB' )) or 
            (name.startswith( 'GL_' ) and name.endswith( 'ARB' ))
        ) and (name != 'glInitVertexBufferObjectARB')
    def basename( self, name ):
        if name.endswith( '_ARB' ):
            return name[:-4]
        elif name.endswith( 'ARB' ):
            return name[:-3]
        else:
            return name
    def __init__( self ):
        names = [name for name in dir(vertex_buffer_object) if self._arbname( name )]
        if not GL.glBufferData:
            for name in names:
                setattr( self, self.basename(name), getattr( vertex_buffer_object, name ))
            self.available = True
        elif vertex_buffer_object.glBufferDataARB:
            for name in names:
                setattr( self, self.basename(name), getattr( GL, self.basename(name) ))
            self.available = True
    def __nonzero__( self ):
        return self.available
    def deleter( self, buffers, key):
        """Produce a deleter callback to delete the given buffer"""
        nfe = error.NullFunctionError
        gluint = constants.GLuint
        def doBufferDeletion( *args, **named ):
            while buffers:
                try:
                    buffer = buffers.pop()
                except IndexError, err:
                    break
                else:
                    try:
                        # Note that to avoid ERROR_ON_COPY issues 
                        # we have to pass an array-compatible type here...
                        buf = gluint( buffer )
                        self.glDeleteBuffers(1, buf)
                    except (AttributeError, nfe), err:
                        pass
            try:
                self._DELETERS_.pop( key )
            except KeyError, err:
                pass
        return doBufferDeletion
    _DELETERS_ = {}

IMPLEMENTATION = None

def get_implementation( *args ):
    """Retrieve the appropriate implementation for this machine"""
    global IMPLEMENTATION
    if IMPLEMENTATION is None:
        IMPLEMENTATION = Implementation()
    return IMPLEMENTATION

from OpenGL import acceleratesupport
VBO = None
if acceleratesupport.ACCELERATE_AVAILABLE:
    try:
        from OpenGL_accelerate.vbo import (
            VBO,VBOOffset,VBOHandler,VBOOffsetHandler,
        )
    except ImportError, err:
        log.warn(
            "Unable to load VBO accelerator from OpenGL_accelerate"
        )
if VBO is None:
    class VBO( object ):
        """Instances can be passed into array-handling routines
        
        You can check for whether VBOs are supported by accessing the implementation
        attribute of the VBO, which will raise a RuntimeError if there is no available 
        implementation.
        """
        copied = False
        _no_cache_ = True # do not cache in context data arrays
        def __init__( 
            self, data, usage='GL_DYNAMIC_DRAW', 
            target='GL_ARRAY_BUFFER', size=None,
        ):
            self.usage = usage 
            self.set_array( data, size )
            self.target = target 
            self.buffers = []
            self._copy_segments = []
        _I_ = None
        implementation = property( get_implementation, )
        def resolve( self, value ):
            """Resolve string constant to constant"""
            if isinstance( value, (str,unicode)):
                return getattr( self.implementation, self.implementation.basename( value ) )
            return value
        def set_array( self, data, size=None ):
            """Update our entire array with new data"""
            self.data = data 
            self.copied = False
            if size is not None:
                self.size = size
            elif self.data is not None:
                self.size = ArrayDatatype.arrayByteCount( self.data )
        def __setitem__( self, slice, array):
            """Set slice of data on the array and vbo (if copied already)
            
            slice -- the Python slice object determining how the data should 
                be copied into the vbo/array 
            array -- something array-compatible that will be used as the 
                source of the data, note that the data-format will have to 
                be the same as the internal data-array to work properly, if 
                not, the amount of data copied will be wrong.
            
            This is a reasonably complex operation, it has to have all sorts
            of state-aware changes to correctly map the source into the low-level
            OpenGL view of the buffer (which is just bytes as far as the GL 
            is concerned).
            """
            if slice.step and not slice.step == 1:
                raise NotImplemented( """Don't know how to map stepped arrays yet""" )
            # TODO: handle e.g. mapping character data into an integer data-set
            data = ArrayDatatype.asArray( array )
            start = (slice.start or 0) 
            stop = (slice.stop or len(self.data))
            if start < 0:
                start += len(self.data)
                start = max((start,0))
            if stop < 0:
                stop += len(self.data)
                stop = max((stop,0))
            self.data[ slice ] = data
            if self.copied and self.buffers:
                if start-stop != len(data):
                    self.copied = False
                elif start-stop == len(self.data):
                    # re-copy the whole data-set
                    self.copied = False
                elif len(data):
                    # now the fun part, we need to make the array match the 
                    # structure of the array we're going to copy into and make 
                    # the "size" parameter match the value we're going to copy in,
                    # note that a 2D array (rather than a 1D array) may require 
                    # multiple mappings to copy into the memory area...
                    
                    # find the step size from the dimensions and base size...
                    size = ArrayDatatype.arrayByteCount( data ) / len(array)
                    #baseSize = ArrayDatatype.unitSize( data )
                    # now create the start and distance values...
                    start *= size
                    stop *= size
                    # wait until the last moment (bind) to copy the data...
                    self._copy_segments.append(
                        (start,(stop-start), data)
                    )
        def __len__( self ):
            return len( self.data )
        def __getattr__( self, key ):
            if key not in ('data','usage','target','buffers', 'copied','_I_','implementation','_copy_segments' ):
                return getattr( self.data, key )
            else:
                raise AttributeError( key )
        def create_buffers( self ):
            """Create the internal buffer(s)"""
            assert not self.buffers, """Already created the buffer"""
            self.buffers = [ long(self.implementation.glGenBuffers(1)) ]
            self.target = self.resolve( self.target )
            self.usage = self.resolve( self.usage )
            self.implementation._DELETERS_[ id(self) ] = weakref.ref( self, self.implementation.deleter( self.buffers, id(self) ))
            return self.buffers
        def copy_data( self ):
            """Copy our data into the buffer on the GL side"""
            assert self.buffers, """Should do create_buffers before copy_data"""
            if self.copied:
                if self._copy_segments:
                    while self._copy_segments:
                        start,size,data  = self._copy_segments.pop(0)
                        dataptr = ArrayDatatype.voidDataPointer( data )
                        self.implementation.glBufferSubData(self.target, start, size, dataptr)
            else:
                if self.data is not None and self.size is None:
                    self.size = ArrayDatatype.arrayByteCount( self.data )
                self.implementation.glBufferData(
                    self.target, 
                    self.size,
                    self.data,
                    self.usage,
                )
                self.copied = True
        def delete( self ):
            """Delete this buffer explicitly"""
            if self.buffers:
                while self.buffers:
                    try:
                        self.implementation.glDeleteBuffers(1, self.buffers.pop(0))
                    except (AttributeError,error.NullFunctionError), err:
                        pass
        def bind( self ):
            """Bind this buffer for use in vertex calls"""
            if not self.buffers:
                buffers = self.create_buffers()
            self.implementation.glBindBuffer( self.target, self.buffers[0])
            self.copy_data()
        def unbind( self ):
            """Unbind the buffer (make normal array operations active)"""
            self.implementation.glBindBuffer( self.target,0 )
        
        def __add__( self, other ):
            """Add an integer to this VBO (offset)"""
            if hasattr( other, 'offset' ):
                other = other.offset 
            assert isinstance( other, (int,long) ), """Only know how to add integer/long offsets"""
            return VBOOffset( self, other )
        
        __enter__ = bind
        def __exit__( self, exc_type=None, exc_val=None, exc_tb=None ):
            """Context manager exit"""
            self.unbind()
            return False # do not supress exceptions...

    class VBOOffset( object ):
        def __init__( self, vbo, offset ):
            self.vbo = vbo 
            self.offset = offset 
        def __getattr__( self, key ):
            if key != 'vbo':
                return getattr( self.vbo, key )
            raise AttributeError( 'No %r key in VBOOffset'%(key,))
        def __add__( self, other ):
            if hasattr( other, 'offset' ):
                other = other.offset 
            return VBOOffset( self.vbo, self.offset + other )


    class VBOHandler( FormatHandler ):
        """Handles VBO instances passed in as array data"""
        vp0 = ctypes.c_void_p( 0 )
        def dataPointer( self, instance ):
            """Retrieve data-pointer from the instance's data
            
            Is always NULL, to indicate use of the bound pointer
            """
            return 0
        def from_param( self, instance, typeCode=None ):
            return self.vp0
        def zeros( self, dims, typeCode ):
            """Not implemented"""
            raise NotImplemented( """Don't have VBO output support yet""" )
        ones = zeros 
        def asArray( self, value, typeCode=None ):
            """Given a value, convert to array representation"""
            return value
        def arrayToGLType( self, value ):
            """Given a value, guess OpenGL type of the corresponding pointer"""
            return ArrayDatatype.arrayToGLType( value.data )
        def arrayByteCount( self, value ):
            return ArrayDatatype.arrayByteCount( value.data )
        def arraySize( self, value, typeCode = None ):
            """Given a data-value, calculate dimensions for the array"""
            return ArrayDatatype.arraySize( value.data )
        def unitSize( self, value, typeCode=None ):
            """Determine unit size of an array (if possible)"""
            return ArrayDatatype.unitSize( value.data )
        def dimensions( self, value, typeCode=None ):
            """Determine dimensions of the passed array value (if possible)"""
            return ArrayDatatype.dimensions( value.data )

    class VBOOffsetHandler( VBOHandler ):
        def dataPointer( self, instance ):
            """Retrieve data-pointer from the instance's data
            
            Is always NULL, to indicate use of the bound pointer
            """
            return instance.offset
        def from_param( self, instance, typeCode=None ):
            return ctypes.c_void_p( instance.offset )



FormatHandler.loadAll() # otherwise just the VBO would get loaded :)
VBO_HANDLER = VBOHandler()
VBO_HANDLER.register( [ VBO ] )

VBOOFFSET_HANDLER = VBOOffsetHandler()
VBOOFFSET_HANDLER.register( [ VBOOffset ] )

PyBuffer_FromMemory = ctypes.pythonapi.PyBuffer_FromMemory
PyBuffer_FromMemory.restype = ctypes.py_object
_cleaners = {}
def _cleaner( vbo ):
    def clean( ref ):
        try:
            _cleaners.pop( vbo )
        except Exception, err:
            pass
        else:
            glUnmapBuffer( vbo.target )
    return clean

def mapVBO( vbo, access=GL.GL_READ_WRITE ):
    """Map the given buffer into a numpy array...
    
    Method taken from:
     http://www.mail-archive.com/numpy-discussion@lists.sourceforge.net/msg01161.html
     
    This should be considered an *experimental* API,
    it is not guaranteed to be available in future revisions 
    of this library!
    """
    vp = glMapBuffer( vbo.target, access )
    buffer = PyBuffer_FromMemory( 
        ctypes.c_void_p(vp), vbo.size 
    )
    array = frombuffer( buffer, 'B' )
    _cleaners[vbo] = weakref.ref( array, _cleaner( vbo ))
    return array



#! /usr/bin/env python
"""Test that GLUT.glutInit accepts 0 arguments"""
from OpenGL import GLUT

if __name__ == "__main__":
    try:
        GLUT.glutInit()
    except Exception, err:
        raise
    else:
        print 'accepted 0 arguments, as desired'
    


'''OpenGL extension NV.packed_depth_stencil

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_NV_packed_depth_stencil'
_DEPRECATED = False
GL_DEPTH_STENCIL_NV = constant.Constant( 'GL_DEPTH_STENCIL_NV', 0x84F9 )
GL_UNSIGNED_INT_24_8_NV = constant.Constant( 'GL_UNSIGNED_INT_24_8_NV', 0x84FA )


def glInitPackedDepthStencilNV():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension DFX.texture_compression_FXT1

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_DFX_texture_compression_FXT1'
_DEPRECATED = False
GL_COMPRESSED_RGB_FXT1_3DFX = constant.Constant( 'GL_COMPRESSED_RGB_FXT1_3DFX', 0x86B0 )
GL_COMPRESSED_RGBA_FXT1_3DFX = constant.Constant( 'GL_COMPRESSED_RGBA_FXT1_3DFX', 0x86B1 )


def glInitTextureCompressionFxt1DFX():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension VERSION.GL_1_2_DEPRECATED

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_VERSION_GL_1_2'
_DEPRECATED = True
GL_RESCALE_NORMAL = constant.Constant( 'GL_RESCALE_NORMAL', 0x803A )
GL_LIGHT_MODEL_COLOR_CONTROL = constant.Constant( 'GL_LIGHT_MODEL_COLOR_CONTROL', 0x81F8 )
GL_SINGLE_COLOR = constant.Constant( 'GL_SINGLE_COLOR', 0x81F9 )
GL_SEPARATE_SPECULAR_COLOR = constant.Constant( 'GL_SEPARATE_SPECULAR_COLOR', 0x81FA )
GL_ALIASED_POINT_SIZE_RANGE = constant.Constant( 'GL_ALIASED_POINT_SIZE_RANGE', 0x846D )
glColorTable = platform.createExtensionFunction( 
'glColorTable',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,constants.GLsizei,constants.GLenum,constants.GLenum,ctypes.c_void_p,),
doc='glColorTable(GLenum(target), GLenum(internalformat), GLsizei(width), GLenum(format), GLenum(type), c_void_p(table)) -> None',
argNames=('target','internalformat','width','format','type','table',),
deprecated=_DEPRECATED,
)

glColorTableParameterfv = platform.createExtensionFunction( 
'glColorTableParameterfv',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,arrays.GLfloatArray,),
doc='glColorTableParameterfv(GLenum(target), GLenum(pname), GLfloatArray(params)) -> None',
argNames=('target','pname','params',),
deprecated=_DEPRECATED,
)

glColorTableParameteriv = platform.createExtensionFunction( 
'glColorTableParameteriv',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,arrays.GLintArray,),
doc='glColorTableParameteriv(GLenum(target), GLenum(pname), GLintArray(params)) -> None',
argNames=('target','pname','params',),
deprecated=_DEPRECATED,
)

glCopyColorTable = platform.createExtensionFunction( 
'glCopyColorTable',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,constants.GLint,constants.GLint,constants.GLsizei,),
doc='glCopyColorTable(GLenum(target), GLenum(internalformat), GLint(x), GLint(y), GLsizei(width)) -> None',
argNames=('target','internalformat','x','y','width',),
deprecated=_DEPRECATED,
)

glGetColorTable = platform.createExtensionFunction( 
'glGetColorTable',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,constants.GLenum,ctypes.c_void_p,),
doc='glGetColorTable(GLenum(target), GLenum(format), GLenum(type), c_void_p(table)) -> None',
argNames=('target','format','type','table',),
deprecated=_DEPRECATED,
)

glGetColorTableParameterfv = platform.createExtensionFunction( 
'glGetColorTableParameterfv',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,arrays.GLfloatArray,),
doc='glGetColorTableParameterfv(GLenum(target), GLenum(pname), GLfloatArray(params)) -> None',
argNames=('target','pname','params',),
deprecated=_DEPRECATED,
)

glGetColorTableParameteriv = platform.createExtensionFunction( 
'glGetColorTableParameteriv',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,arrays.GLintArray,),
doc='glGetColorTableParameteriv(GLenum(target), GLenum(pname), GLintArray(params)) -> None',
argNames=('target','pname','params',),
deprecated=_DEPRECATED,
)

glColorSubTable = platform.createExtensionFunction( 
'glColorSubTable',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLsizei,constants.GLsizei,constants.GLenum,constants.GLenum,ctypes.c_void_p,),
doc='glColorSubTable(GLenum(target), GLsizei(start), GLsizei(count), GLenum(format), GLenum(type), c_void_p(data)) -> None',
argNames=('target','start','count','format','type','data',),
deprecated=_DEPRECATED,
)

glCopyColorSubTable = platform.createExtensionFunction( 
'glCopyColorSubTable',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLsizei,constants.GLint,constants.GLint,constants.GLsizei,),
doc='glCopyColorSubTable(GLenum(target), GLsizei(start), GLint(x), GLint(y), GLsizei(width)) -> None',
argNames=('target','start','x','y','width',),
deprecated=_DEPRECATED,
)

glConvolutionFilter1D = platform.createExtensionFunction( 
'glConvolutionFilter1D',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,constants.GLsizei,constants.GLenum,constants.GLenum,ctypes.c_void_p,),
doc='glConvolutionFilter1D(GLenum(target), GLenum(internalformat), GLsizei(width), GLenum(format), GLenum(type), c_void_p(image)) -> None',
argNames=('target','internalformat','width','format','type','image',),
deprecated=_DEPRECATED,
)

glConvolutionFilter2D = platform.createExtensionFunction( 
'glConvolutionFilter2D',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,constants.GLsizei,constants.GLsizei,constants.GLenum,constants.GLenum,ctypes.c_void_p,),
doc='glConvolutionFilter2D(GLenum(target), GLenum(internalformat), GLsizei(width), GLsizei(height), GLenum(format), GLenum(type), c_void_p(image)) -> None',
argNames=('target','internalformat','width','height','format','type','image',),
deprecated=_DEPRECATED,
)

glConvolutionParameterf = platform.createExtensionFunction( 
'glConvolutionParameterf',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,constants.GLfloat,),
doc='glConvolutionParameterf(GLenum(target), GLenum(pname), GLfloat(params)) -> None',
argNames=('target','pname','params',),
deprecated=_DEPRECATED,
)

glConvolutionParameterfv = platform.createExtensionFunction( 
'glConvolutionParameterfv',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,arrays.GLfloatArray,),
doc='glConvolutionParameterfv(GLenum(target), GLenum(pname), GLfloatArray(params)) -> None',
argNames=('target','pname','params',),
deprecated=_DEPRECATED,
)

glConvolutionParameteri = platform.createExtensionFunction( 
'glConvolutionParameteri',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,constants.GLint,),
doc='glConvolutionParameteri(GLenum(target), GLenum(pname), GLint(params)) -> None',
argNames=('target','pname','params',),
deprecated=_DEPRECATED,
)

glConvolutionParameteriv = platform.createExtensionFunction( 
'glConvolutionParameteriv',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,arrays.GLintArray,),
doc='glConvolutionParameteriv(GLenum(target), GLenum(pname), GLintArray(params)) -> None',
argNames=('target','pname','params',),
deprecated=_DEPRECATED,
)

glCopyConvolutionFilter1D = platform.createExtensionFunction( 
'glCopyConvolutionFilter1D',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,constants.GLint,constants.GLint,constants.GLsizei,),
doc='glCopyConvolutionFilter1D(GLenum(target), GLenum(internalformat), GLint(x), GLint(y), GLsizei(width)) -> None',
argNames=('target','internalformat','x','y','width',),
deprecated=_DEPRECATED,
)

glCopyConvolutionFilter2D = platform.createExtensionFunction( 
'glCopyConvolutionFilter2D',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,constants.GLint,constants.GLint,constants.GLsizei,constants.GLsizei,),
doc='glCopyConvolutionFilter2D(GLenum(target), GLenum(internalformat), GLint(x), GLint(y), GLsizei(width), GLsizei(height)) -> None',
argNames=('target','internalformat','x','y','width','height',),
deprecated=_DEPRECATED,
)

glGetConvolutionFilter = platform.createExtensionFunction( 
'glGetConvolutionFilter',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,constants.GLenum,ctypes.c_void_p,),
doc='glGetConvolutionFilter(GLenum(target), GLenum(format), GLenum(type), c_void_p(image)) -> None',
argNames=('target','format','type','image',),
deprecated=_DEPRECATED,
)

glGetConvolutionParameterfv = platform.createExtensionFunction( 
'glGetConvolutionParameterfv',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,arrays.GLfloatArray,),
doc='glGetConvolutionParameterfv(GLenum(target), GLenum(pname), GLfloatArray(params)) -> None',
argNames=('target','pname','params',),
deprecated=_DEPRECATED,
)

glGetConvolutionParameteriv = platform.createExtensionFunction( 
'glGetConvolutionParameteriv',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,arrays.GLintArray,),
doc='glGetConvolutionParameteriv(GLenum(target), GLenum(pname), GLintArray(params)) -> None',
argNames=('target','pname','params',),
deprecated=_DEPRECATED,
)

glGetSeparableFilter = platform.createExtensionFunction( 
'glGetSeparableFilter',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,constants.GLenum,ctypes.c_void_p,ctypes.c_void_p,ctypes.c_void_p,),
doc='glGetSeparableFilter(GLenum(target), GLenum(format), GLenum(type), c_void_p(row), c_void_p(column), c_void_p(span)) -> None',
argNames=('target','format','type','row','column','span',),
deprecated=_DEPRECATED,
)

glSeparableFilter2D = platform.createExtensionFunction( 
'glSeparableFilter2D',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,constants.GLsizei,constants.GLsizei,constants.GLenum,constants.GLenum,ctypes.c_void_p,ctypes.c_void_p,),
doc='glSeparableFilter2D(GLenum(target), GLenum(internalformat), GLsizei(width), GLsizei(height), GLenum(format), GLenum(type), c_void_p(row), c_void_p(column)) -> None',
argNames=('target','internalformat','width','height','format','type','row','column',),
deprecated=_DEPRECATED,
)

glGetHistogram = platform.createExtensionFunction( 
'glGetHistogram',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLboolean,constants.GLenum,constants.GLenum,ctypes.c_void_p,),
doc='glGetHistogram(GLenum(target), GLboolean(reset), GLenum(format), GLenum(type), c_void_p(values)) -> None',
argNames=('target','reset','format','type','values',),
deprecated=_DEPRECATED,
)

glGetHistogramParameterfv = platform.createExtensionFunction( 
'glGetHistogramParameterfv',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,arrays.GLfloatArray,),
doc='glGetHistogramParameterfv(GLenum(target), GLenum(pname), GLfloatArray(params)) -> None',
argNames=('target','pname','params',),
deprecated=_DEPRECATED,
)

glGetHistogramParameteriv = platform.createExtensionFunction( 
'glGetHistogramParameteriv',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,arrays.GLintArray,),
doc='glGetHistogramParameteriv(GLenum(target), GLenum(pname), GLintArray(params)) -> None',
argNames=('target','pname','params',),
deprecated=_DEPRECATED,
)

glGetMinmax = platform.createExtensionFunction( 
'glGetMinmax',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLboolean,constants.GLenum,constants.GLenum,ctypes.c_void_p,),
doc='glGetMinmax(GLenum(target), GLboolean(reset), GLenum(format), GLenum(type), c_void_p(values)) -> None',
argNames=('target','reset','format','type','values',),
deprecated=_DEPRECATED,
)

glGetMinmaxParameterfv = platform.createExtensionFunction( 
'glGetMinmaxParameterfv',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,arrays.GLfloatArray,),
doc='glGetMinmaxParameterfv(GLenum(target), GLenum(pname), GLfloatArray(params)) -> None',
argNames=('target','pname','params',),
deprecated=_DEPRECATED,
)

glGetMinmaxParameteriv = platform.createExtensionFunction( 
'glGetMinmaxParameteriv',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,arrays.GLintArray,),
doc='glGetMinmaxParameteriv(GLenum(target), GLenum(pname), GLintArray(params)) -> None',
argNames=('target','pname','params',),
deprecated=_DEPRECATED,
)

glHistogram = platform.createExtensionFunction( 
'glHistogram',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLsizei,constants.GLenum,constants.GLboolean,),
doc='glHistogram(GLenum(target), GLsizei(width), GLenum(internalformat), GLboolean(sink)) -> None',
argNames=('target','width','internalformat','sink',),
deprecated=_DEPRECATED,
)

glMinmax = platform.createExtensionFunction( 
'glMinmax',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,constants.GLboolean,),
doc='glMinmax(GLenum(target), GLenum(internalformat), GLboolean(sink)) -> None',
argNames=('target','internalformat','sink',),
deprecated=_DEPRECATED,
)

glResetHistogram = platform.createExtensionFunction( 
'glResetHistogram',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,),
doc='glResetHistogram(GLenum(target)) -> None',
argNames=('target',),
deprecated=_DEPRECATED,
)

glResetMinmax = platform.createExtensionFunction( 
'glResetMinmax',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,),
doc='glResetMinmax(GLenum(target)) -> None',
argNames=('target',),
deprecated=_DEPRECATED,
)




if __name__ == "__main__":
    import pygame
    pygame.init()
    screen = pygame.display.set_mode([800, 600], pygame.OPENGL | pygame.DOUBLEBUF)
    from OpenGL.GL import *
    from OpenGL.GLU import *


'''OpenGL extension ARB.geometry_shader4

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_ARB_geometry_shader4'
_DEPRECATED = False
GL_LINES_ADJACENCY_ARB = constant.Constant( 'GL_LINES_ADJACENCY_ARB', 0xA )
GL_LINE_STRIP_ADJACENCY_ARB = constant.Constant( 'GL_LINE_STRIP_ADJACENCY_ARB', 0xB )
GL_TRIANGLES_ADJACENCY_ARB = constant.Constant( 'GL_TRIANGLES_ADJACENCY_ARB', 0xC )
GL_TRIANGLE_STRIP_ADJACENCY_ARB = constant.Constant( 'GL_TRIANGLE_STRIP_ADJACENCY_ARB', 0xD )
GL_PROGRAM_POINT_SIZE_ARB = constant.Constant( 'GL_PROGRAM_POINT_SIZE_ARB', 0x8642 )
glget.addGLGetConstant( GL_PROGRAM_POINT_SIZE_ARB, (1,) )
GL_MAX_GEOMETRY_TEXTURE_IMAGE_UNITS_ARB = constant.Constant( 'GL_MAX_GEOMETRY_TEXTURE_IMAGE_UNITS_ARB', 0x8C29 )
glget.addGLGetConstant( GL_MAX_GEOMETRY_TEXTURE_IMAGE_UNITS_ARB, (1,) )
GL_FRAMEBUFFER_ATTACHMENT_LAYERED_ARB = constant.Constant( 'GL_FRAMEBUFFER_ATTACHMENT_LAYERED_ARB', 0x8DA7 )
GL_FRAMEBUFFER_INCOMPLETE_LAYER_TARGETS_ARB = constant.Constant( 'GL_FRAMEBUFFER_INCOMPLETE_LAYER_TARGETS_ARB', 0x8DA8 )
GL_FRAMEBUFFER_INCOMPLETE_LAYER_COUNT_ARB = constant.Constant( 'GL_FRAMEBUFFER_INCOMPLETE_LAYER_COUNT_ARB', 0x8DA9 )
GL_GEOMETRY_SHADER_ARB = constant.Constant( 'GL_GEOMETRY_SHADER_ARB', 0x8DD9 )
GL_GEOMETRY_VERTICES_OUT_ARB = constant.Constant( 'GL_GEOMETRY_VERTICES_OUT_ARB', 0x8DDA )
GL_GEOMETRY_INPUT_TYPE_ARB = constant.Constant( 'GL_GEOMETRY_INPUT_TYPE_ARB', 0x8DDB )
GL_GEOMETRY_OUTPUT_TYPE_ARB = constant.Constant( 'GL_GEOMETRY_OUTPUT_TYPE_ARB', 0x8DDC )
GL_MAX_GEOMETRY_VARYING_COMPONENTS_ARB = constant.Constant( 'GL_MAX_GEOMETRY_VARYING_COMPONENTS_ARB', 0x8DDD )
glget.addGLGetConstant( GL_MAX_GEOMETRY_VARYING_COMPONENTS_ARB, (1,) )
GL_MAX_VERTEX_VARYING_COMPONENTS_ARB = constant.Constant( 'GL_MAX_VERTEX_VARYING_COMPONENTS_ARB', 0x8DDE )
glget.addGLGetConstant( GL_MAX_VERTEX_VARYING_COMPONENTS_ARB, (1,) )
GL_MAX_GEOMETRY_UNIFORM_COMPONENTS_ARB = constant.Constant( 'GL_MAX_GEOMETRY_UNIFORM_COMPONENTS_ARB', 0x8DDF )
glget.addGLGetConstant( GL_MAX_GEOMETRY_UNIFORM_COMPONENTS_ARB, (1,) )
GL_MAX_GEOMETRY_OUTPUT_VERTICES_ARB = constant.Constant( 'GL_MAX_GEOMETRY_OUTPUT_VERTICES_ARB', 0x8DE0 )
glget.addGLGetConstant( GL_MAX_GEOMETRY_OUTPUT_VERTICES_ARB, (1,) )
GL_MAX_GEOMETRY_TOTAL_OUTPUT_COMPONENTS_ARB = constant.Constant( 'GL_MAX_GEOMETRY_TOTAL_OUTPUT_COMPONENTS_ARB', 0x8DE1 )
glget.addGLGetConstant( GL_MAX_GEOMETRY_TOTAL_OUTPUT_COMPONENTS_ARB, (1,) )
glProgramParameteriARB = platform.createExtensionFunction( 
'glProgramParameteriARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLenum,constants.GLint,),
doc='glProgramParameteriARB(GLuint(program), GLenum(pname), GLint(value)) -> None',
argNames=('program','pname','value',),
deprecated=_DEPRECATED,
)

glFramebufferTextureARB = platform.createExtensionFunction( 
'glFramebufferTextureARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,constants.GLuint,constants.GLint,),
doc='glFramebufferTextureARB(GLenum(target), GLenum(attachment), GLuint(texture), GLint(level)) -> None',
argNames=('target','attachment','texture','level',),
deprecated=_DEPRECATED,
)

glFramebufferTextureLayerARB = platform.createExtensionFunction( 
'glFramebufferTextureLayerARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,constants.GLuint,constants.GLint,constants.GLint,),
doc='glFramebufferTextureLayerARB(GLenum(target), GLenum(attachment), GLuint(texture), GLint(level), GLint(layer)) -> None',
argNames=('target','attachment','texture','level','layer',),
deprecated=_DEPRECATED,
)

glFramebufferTextureFaceARB = platform.createExtensionFunction( 
'glFramebufferTextureFaceARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,constants.GLuint,constants.GLint,constants.GLenum,),
doc='glFramebufferTextureFaceARB(GLenum(target), GLenum(attachment), GLuint(texture), GLint(level), GLenum(face)) -> None',
argNames=('target','attachment','texture','level','face',),
deprecated=_DEPRECATED,
)


def glInitGeometryShader4ARB():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension EXT.stencil_wrap

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_stencil_wrap'
_DEPRECATED = False
GL_INCR_WRAP_EXT = constant.Constant( 'GL_INCR_WRAP_EXT', 0x8507 )
GL_DECR_WRAP_EXT = constant.Constant( 'GL_DECR_WRAP_EXT', 0x8508 )


def glInitStencilWrapEXT():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension NV.conditional_render

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_NV_conditional_render'
_DEPRECATED = False
GL_QUERY_WAIT_NV = constant.Constant( 'GL_QUERY_WAIT_NV', 0x8E13 )
GL_QUERY_NO_WAIT_NV = constant.Constant( 'GL_QUERY_NO_WAIT_NV', 0x8E14 )
GL_QUERY_BY_REGION_WAIT_NV = constant.Constant( 'GL_QUERY_BY_REGION_WAIT_NV', 0x8E15 )
GL_QUERY_BY_REGION_NO_WAIT_NV = constant.Constant( 'GL_QUERY_BY_REGION_NO_WAIT_NV', 0x8E16 )
glBeginConditionalRenderNV = platform.createExtensionFunction( 
'glBeginConditionalRenderNV',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLenum,),
doc='glBeginConditionalRenderNV(GLuint(id), GLenum(mode)) -> None',
argNames=('id','mode',),
deprecated=_DEPRECATED,
)

glEndConditionalRenderNV = platform.createExtensionFunction( 
'glEndConditionalRenderNV',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(),
doc='glEndConditionalRenderNV() -> None',
argNames=(),
deprecated=_DEPRECATED,
)


def glInitConditionalRenderNV():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension EXT.texture_snorm

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_texture_snorm'
_DEPRECATED = False
GL_ALPHA_SNORM = constant.Constant( 'GL_ALPHA_SNORM', 0x9010 )
GL_LUMINANCE_SNORM = constant.Constant( 'GL_LUMINANCE_SNORM', 0x9011 )
GL_LUMINANCE_ALPHA_SNORM = constant.Constant( 'GL_LUMINANCE_ALPHA_SNORM', 0x9012 )
GL_INTENSITY_SNORM = constant.Constant( 'GL_INTENSITY_SNORM', 0x9013 )
GL_ALPHA8_SNORM = constant.Constant( 'GL_ALPHA8_SNORM', 0x9014 )
GL_LUMINANCE8_SNORM = constant.Constant( 'GL_LUMINANCE8_SNORM', 0x9015 )
GL_LUMINANCE8_ALPHA8_SNORM = constant.Constant( 'GL_LUMINANCE8_ALPHA8_SNORM', 0x9016 )
GL_INTENSITY8_SNORM = constant.Constant( 'GL_INTENSITY8_SNORM', 0x9017 )
GL_ALPHA16_SNORM = constant.Constant( 'GL_ALPHA16_SNORM', 0x9018 )
GL_LUMINANCE16_SNORM = constant.Constant( 'GL_LUMINANCE16_SNORM', 0x9019 )
GL_LUMINANCE16_ALPHA16_SNORM = constant.Constant( 'GL_LUMINANCE16_ALPHA16_SNORM', 0x901A )
GL_INTENSITY16_SNORM = constant.Constant( 'GL_INTENSITY16_SNORM', 0x901B )


def glInitTextureSnormEXT():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension ARB.point_parameters

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_ARB_point_parameters'
_DEPRECATED = False
GL_POINT_SIZE_MIN_ARB = constant.Constant( 'GL_POINT_SIZE_MIN_ARB', 0x8126 )
glget.addGLGetConstant( GL_POINT_SIZE_MIN_ARB, (1,) )
GL_POINT_SIZE_MAX_ARB = constant.Constant( 'GL_POINT_SIZE_MAX_ARB', 0x8127 )
glget.addGLGetConstant( GL_POINT_SIZE_MAX_ARB, (1,) )
GL_POINT_FADE_THRESHOLD_SIZE_ARB = constant.Constant( 'GL_POINT_FADE_THRESHOLD_SIZE_ARB', 0x8128 )
glget.addGLGetConstant( GL_POINT_FADE_THRESHOLD_SIZE_ARB, (1,) )
GL_POINT_DISTANCE_ATTENUATION_ARB = constant.Constant( 'GL_POINT_DISTANCE_ATTENUATION_ARB', 0x8129 )
glget.addGLGetConstant( GL_POINT_DISTANCE_ATTENUATION_ARB, (3,) )
glPointParameterfARB = platform.createExtensionFunction( 
'glPointParameterfARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLfloat,),
doc='glPointParameterfARB(GLenum(pname), GLfloat(param)) -> None',
argNames=('pname','param',),
deprecated=_DEPRECATED,
)

glPointParameterfvARB = platform.createExtensionFunction( 
'glPointParameterfvARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,arrays.GLfloatArray,),
doc='glPointParameterfvARB(GLenum(pname), GLfloatArray(params)) -> None',
argNames=('pname','params',),
deprecated=_DEPRECATED,
)


def glInitPointParametersARB():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension ARB.shader_objects

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_ARB_shader_objects'
_DEPRECATED = False
GL_PROGRAM_OBJECT_ARB = constant.Constant( 'GL_PROGRAM_OBJECT_ARB', 0x8B40 )
GL_SHADER_OBJECT_ARB = constant.Constant( 'GL_SHADER_OBJECT_ARB', 0x8B48 )
GL_OBJECT_TYPE_ARB = constant.Constant( 'GL_OBJECT_TYPE_ARB', 0x8B4E )
GL_OBJECT_SUBTYPE_ARB = constant.Constant( 'GL_OBJECT_SUBTYPE_ARB', 0x8B4F )
GL_FLOAT_VEC2_ARB = constant.Constant( 'GL_FLOAT_VEC2_ARB', 0x8B50 )
GL_FLOAT_VEC3_ARB = constant.Constant( 'GL_FLOAT_VEC3_ARB', 0x8B51 )
GL_FLOAT_VEC4_ARB = constant.Constant( 'GL_FLOAT_VEC4_ARB', 0x8B52 )
GL_INT_VEC2_ARB = constant.Constant( 'GL_INT_VEC2_ARB', 0x8B53 )
GL_INT_VEC3_ARB = constant.Constant( 'GL_INT_VEC3_ARB', 0x8B54 )
GL_INT_VEC4_ARB = constant.Constant( 'GL_INT_VEC4_ARB', 0x8B55 )
GL_BOOL_ARB = constant.Constant( 'GL_BOOL_ARB', 0x8B56 )
GL_BOOL_VEC2_ARB = constant.Constant( 'GL_BOOL_VEC2_ARB', 0x8B57 )
GL_BOOL_VEC3_ARB = constant.Constant( 'GL_BOOL_VEC3_ARB', 0x8B58 )
GL_BOOL_VEC4_ARB = constant.Constant( 'GL_BOOL_VEC4_ARB', 0x8B59 )
GL_FLOAT_MAT2_ARB = constant.Constant( 'GL_FLOAT_MAT2_ARB', 0x8B5A )
GL_FLOAT_MAT3_ARB = constant.Constant( 'GL_FLOAT_MAT3_ARB', 0x8B5B )
GL_FLOAT_MAT4_ARB = constant.Constant( 'GL_FLOAT_MAT4_ARB', 0x8B5C )
GL_SAMPLER_1D_ARB = constant.Constant( 'GL_SAMPLER_1D_ARB', 0x8B5D )
GL_SAMPLER_2D_ARB = constant.Constant( 'GL_SAMPLER_2D_ARB', 0x8B5E )
GL_SAMPLER_3D_ARB = constant.Constant( 'GL_SAMPLER_3D_ARB', 0x8B5F )
GL_SAMPLER_CUBE_ARB = constant.Constant( 'GL_SAMPLER_CUBE_ARB', 0x8B60 )
GL_SAMPLER_1D_SHADOW_ARB = constant.Constant( 'GL_SAMPLER_1D_SHADOW_ARB', 0x8B61 )
GL_SAMPLER_2D_SHADOW_ARB = constant.Constant( 'GL_SAMPLER_2D_SHADOW_ARB', 0x8B62 )
GL_SAMPLER_2D_RECT_ARB = constant.Constant( 'GL_SAMPLER_2D_RECT_ARB', 0x8B63 )
GL_SAMPLER_2D_RECT_SHADOW_ARB = constant.Constant( 'GL_SAMPLER_2D_RECT_SHADOW_ARB', 0x8B64 )
GL_OBJECT_DELETE_STATUS_ARB = constant.Constant( 'GL_OBJECT_DELETE_STATUS_ARB', 0x8B80 )
GL_OBJECT_COMPILE_STATUS_ARB = constant.Constant( 'GL_OBJECT_COMPILE_STATUS_ARB', 0x8B81 )
GL_OBJECT_LINK_STATUS_ARB = constant.Constant( 'GL_OBJECT_LINK_STATUS_ARB', 0x8B82 )
GL_OBJECT_VALIDATE_STATUS_ARB = constant.Constant( 'GL_OBJECT_VALIDATE_STATUS_ARB', 0x8B83 )
GL_OBJECT_INFO_LOG_LENGTH_ARB = constant.Constant( 'GL_OBJECT_INFO_LOG_LENGTH_ARB', 0x8B84 )
GL_OBJECT_ATTACHED_OBJECTS_ARB = constant.Constant( 'GL_OBJECT_ATTACHED_OBJECTS_ARB', 0x8B85 )
GL_OBJECT_ACTIVE_UNIFORMS_ARB = constant.Constant( 'GL_OBJECT_ACTIVE_UNIFORMS_ARB', 0x8B86 )
GL_OBJECT_ACTIVE_UNIFORM_MAX_LENGTH_ARB = constant.Constant( 'GL_OBJECT_ACTIVE_UNIFORM_MAX_LENGTH_ARB', 0x8B87 )
GL_OBJECT_SHADER_SOURCE_LENGTH_ARB = constant.Constant( 'GL_OBJECT_SHADER_SOURCE_LENGTH_ARB', 0x8B88 )
glDeleteObjectARB = platform.createExtensionFunction( 
'glDeleteObjectARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLhandleARB,),
doc='glDeleteObjectARB(GLhandleARB(obj)) -> None',
argNames=('obj',),
deprecated=_DEPRECATED,
)

glGetHandleARB = platform.createExtensionFunction( 
'glGetHandleARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=constants.GLhandleARB, 
argTypes=(constants.GLenum,),
doc='glGetHandleARB(GLenum(pname)) -> constants.GLhandleARB',
argNames=('pname',),
deprecated=_DEPRECATED,
)

glDetachObjectARB = platform.createExtensionFunction( 
'glDetachObjectARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLhandleARB,constants.GLhandleARB,),
doc='glDetachObjectARB(GLhandleARB(containerObj), GLhandleARB(attachedObj)) -> None',
argNames=('containerObj','attachedObj',),
deprecated=_DEPRECATED,
)

glCreateShaderObjectARB = platform.createExtensionFunction( 
'glCreateShaderObjectARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=constants.GLhandleARB, 
argTypes=(constants.GLenum,),
doc='glCreateShaderObjectARB(GLenum(shaderType)) -> constants.GLhandleARB',
argNames=('shaderType',),
deprecated=_DEPRECATED,
)

glShaderSourceARB = platform.createExtensionFunction( 
'glShaderSourceARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLhandleARB,constants.GLsizei,ctypes.POINTER( ctypes.POINTER( constants.GLchar )),arrays.GLintArray,),
doc='glShaderSourceARB(GLhandleARB(shaderObj), GLsizei(count), POINTER( ctypes.POINTER( constants.GLchar ))(string), GLintArray(length)) -> None',
argNames=('shaderObj','count','string','length',),
deprecated=_DEPRECATED,
)

glCompileShaderARB = platform.createExtensionFunction( 
'glCompileShaderARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLhandleARB,),
doc='glCompileShaderARB(GLhandleARB(shaderObj)) -> None',
argNames=('shaderObj',),
deprecated=_DEPRECATED,
)

glCreateProgramObjectARB = platform.createExtensionFunction( 
'glCreateProgramObjectARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=constants.GLhandleARB, 
argTypes=(),
doc='glCreateProgramObjectARB() -> constants.GLhandleARB',
argNames=(),
deprecated=_DEPRECATED,
)

glAttachObjectARB = platform.createExtensionFunction( 
'glAttachObjectARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLhandleARB,constants.GLhandleARB,),
doc='glAttachObjectARB(GLhandleARB(containerObj), GLhandleARB(obj)) -> None',
argNames=('containerObj','obj',),
deprecated=_DEPRECATED,
)

glLinkProgramARB = platform.createExtensionFunction( 
'glLinkProgramARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLhandleARB,),
doc='glLinkProgramARB(GLhandleARB(programObj)) -> None',
argNames=('programObj',),
deprecated=_DEPRECATED,
)

glUseProgramObjectARB = platform.createExtensionFunction( 
'glUseProgramObjectARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLhandleARB,),
doc='glUseProgramObjectARB(GLhandleARB(programObj)) -> None',
argNames=('programObj',),
deprecated=_DEPRECATED,
)

glValidateProgramARB = platform.createExtensionFunction( 
'glValidateProgramARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLhandleARB,),
doc='glValidateProgramARB(GLhandleARB(programObj)) -> None',
argNames=('programObj',),
deprecated=_DEPRECATED,
)

glUniform1fARB = platform.createExtensionFunction( 
'glUniform1fARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLint,constants.GLfloat,),
doc='glUniform1fARB(GLint(location), GLfloat(v0)) -> None',
argNames=('location','v0',),
deprecated=_DEPRECATED,
)

glUniform2fARB = platform.createExtensionFunction( 
'glUniform2fARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLint,constants.GLfloat,constants.GLfloat,),
doc='glUniform2fARB(GLint(location), GLfloat(v0), GLfloat(v1)) -> None',
argNames=('location','v0','v1',),
deprecated=_DEPRECATED,
)

glUniform3fARB = platform.createExtensionFunction( 
'glUniform3fARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLint,constants.GLfloat,constants.GLfloat,constants.GLfloat,),
doc='glUniform3fARB(GLint(location), GLfloat(v0), GLfloat(v1), GLfloat(v2)) -> None',
argNames=('location','v0','v1','v2',),
deprecated=_DEPRECATED,
)

glUniform4fARB = platform.createExtensionFunction( 
'glUniform4fARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLint,constants.GLfloat,constants.GLfloat,constants.GLfloat,constants.GLfloat,),
doc='glUniform4fARB(GLint(location), GLfloat(v0), GLfloat(v1), GLfloat(v2), GLfloat(v3)) -> None',
argNames=('location','v0','v1','v2','v3',),
deprecated=_DEPRECATED,
)

glUniform1iARB = platform.createExtensionFunction( 
'glUniform1iARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLint,constants.GLint,),
doc='glUniform1iARB(GLint(location), GLint(v0)) -> None',
argNames=('location','v0',),
deprecated=_DEPRECATED,
)

glUniform2iARB = platform.createExtensionFunction( 
'glUniform2iARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLint,constants.GLint,constants.GLint,),
doc='glUniform2iARB(GLint(location), GLint(v0), GLint(v1)) -> None',
argNames=('location','v0','v1',),
deprecated=_DEPRECATED,
)

glUniform3iARB = platform.createExtensionFunction( 
'glUniform3iARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLint,constants.GLint,constants.GLint,constants.GLint,),
doc='glUniform3iARB(GLint(location), GLint(v0), GLint(v1), GLint(v2)) -> None',
argNames=('location','v0','v1','v2',),
deprecated=_DEPRECATED,
)

glUniform4iARB = platform.createExtensionFunction( 
'glUniform4iARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLint,constants.GLint,constants.GLint,constants.GLint,constants.GLint,),
doc='glUniform4iARB(GLint(location), GLint(v0), GLint(v1), GLint(v2), GLint(v3)) -> None',
argNames=('location','v0','v1','v2','v3',),
deprecated=_DEPRECATED,
)

glUniform1fvARB = platform.createExtensionFunction( 
'glUniform1fvARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLint,constants.GLsizei,arrays.GLfloatArray,),
doc='glUniform1fvARB(GLint(location), GLsizei(count), GLfloatArray(value)) -> None',
argNames=('location','count','value',),
deprecated=_DEPRECATED,
)

glUniform2fvARB = platform.createExtensionFunction( 
'glUniform2fvARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLint,constants.GLsizei,arrays.GLfloatArray,),
doc='glUniform2fvARB(GLint(location), GLsizei(count), GLfloatArray(value)) -> None',
argNames=('location','count','value',),
deprecated=_DEPRECATED,
)

glUniform3fvARB = platform.createExtensionFunction( 
'glUniform3fvARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLint,constants.GLsizei,arrays.GLfloatArray,),
doc='glUniform3fvARB(GLint(location), GLsizei(count), GLfloatArray(value)) -> None',
argNames=('location','count','value',),
deprecated=_DEPRECATED,
)

glUniform4fvARB = platform.createExtensionFunction( 
'glUniform4fvARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLint,constants.GLsizei,arrays.GLfloatArray,),
doc='glUniform4fvARB(GLint(location), GLsizei(count), GLfloatArray(value)) -> None',
argNames=('location','count','value',),
deprecated=_DEPRECATED,
)

glUniform1ivARB = platform.createExtensionFunction( 
'glUniform1ivARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLint,constants.GLsizei,arrays.GLintArray,),
doc='glUniform1ivARB(GLint(location), GLsizei(count), GLintArray(value)) -> None',
argNames=('location','count','value',),
deprecated=_DEPRECATED,
)

glUniform2ivARB = platform.createExtensionFunction( 
'glUniform2ivARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLint,constants.GLsizei,arrays.GLintArray,),
doc='glUniform2ivARB(GLint(location), GLsizei(count), GLintArray(value)) -> None',
argNames=('location','count','value',),
deprecated=_DEPRECATED,
)

glUniform3ivARB = platform.createExtensionFunction( 
'glUniform3ivARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLint,constants.GLsizei,arrays.GLintArray,),
doc='glUniform3ivARB(GLint(location), GLsizei(count), GLintArray(value)) -> None',
argNames=('location','count','value',),
deprecated=_DEPRECATED,
)

glUniform4ivARB = platform.createExtensionFunction( 
'glUniform4ivARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLint,constants.GLsizei,arrays.GLintArray,),
doc='glUniform4ivARB(GLint(location), GLsizei(count), GLintArray(value)) -> None',
argNames=('location','count','value',),
deprecated=_DEPRECATED,
)

glUniformMatrix2fvARB = platform.createExtensionFunction( 
'glUniformMatrix2fvARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLint,constants.GLsizei,constants.GLboolean,arrays.GLfloatArray,),
doc='glUniformMatrix2fvARB(GLint(location), GLsizei(count), GLboolean(transpose), GLfloatArray(value)) -> None',
argNames=('location','count','transpose','value',),
deprecated=_DEPRECATED,
)

glUniformMatrix3fvARB = platform.createExtensionFunction( 
'glUniformMatrix3fvARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLint,constants.GLsizei,constants.GLboolean,arrays.GLfloatArray,),
doc='glUniformMatrix3fvARB(GLint(location), GLsizei(count), GLboolean(transpose), GLfloatArray(value)) -> None',
argNames=('location','count','transpose','value',),
deprecated=_DEPRECATED,
)

glUniformMatrix4fvARB = platform.createExtensionFunction( 
'glUniformMatrix4fvARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLint,constants.GLsizei,constants.GLboolean,arrays.GLfloatArray,),
doc='glUniformMatrix4fvARB(GLint(location), GLsizei(count), GLboolean(transpose), GLfloatArray(value)) -> None',
argNames=('location','count','transpose','value',),
deprecated=_DEPRECATED,
)

glGetObjectParameterfvARB = platform.createExtensionFunction( 
'glGetObjectParameterfvARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLhandleARB,constants.GLenum,arrays.GLfloatArray,),
doc='glGetObjectParameterfvARB(GLhandleARB(obj), GLenum(pname), GLfloatArray(params)) -> None',
argNames=('obj','pname','params',),
deprecated=_DEPRECATED,
)

glGetObjectParameterivARB = platform.createExtensionFunction( 
'glGetObjectParameterivARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLhandleARB,constants.GLenum,arrays.GLintArray,),
doc='glGetObjectParameterivARB(GLhandleARB(obj), GLenum(pname), GLintArray(params)) -> None',
argNames=('obj','pname','params',),
deprecated=_DEPRECATED,
)

glGetInfoLogARB = platform.createExtensionFunction( 
'glGetInfoLogARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLhandleARB,constants.GLsizei,arrays.GLsizeiArray,arrays.GLcharARBArray,),
doc='glGetInfoLogARB(GLhandleARB(obj), GLsizei(maxLength), GLsizeiArray(length), GLcharARBArray(infoLog)) -> None',
argNames=('obj','maxLength','length','infoLog',),
deprecated=_DEPRECATED,
)

glGetAttachedObjectsARB = platform.createExtensionFunction( 
'glGetAttachedObjectsARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLhandleARB,constants.GLsizei,arrays.GLsizeiArray,arrays.GLuintArray,),
doc='glGetAttachedObjectsARB(GLhandleARB(containerObj), GLsizei(maxCount), GLsizeiArray(count), GLuintArray(obj)) -> None',
argNames=('containerObj','maxCount','count','obj',),
deprecated=_DEPRECATED,
)

glGetUniformLocationARB = platform.createExtensionFunction( 
'glGetUniformLocationARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=constants.GLint, 
argTypes=(constants.GLhandleARB,arrays.GLcharARBArray,),
doc='glGetUniformLocationARB(GLhandleARB(programObj), GLcharARBArray(name)) -> constants.GLint',
argNames=('programObj','name',),
deprecated=_DEPRECATED,
)

glGetActiveUniformARB = platform.createExtensionFunction( 
'glGetActiveUniformARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLhandleARB,constants.GLuint,constants.GLsizei,arrays.GLsizeiArray,arrays.GLintArray,arrays.GLuintArray,arrays.GLcharARBArray,),
doc='glGetActiveUniformARB(GLhandleARB(programObj), GLuint(index), GLsizei(maxLength), GLsizeiArray(length), GLintArray(size), GLuintArray(type), GLcharARBArray(name)) -> None',
argNames=('programObj','index','maxLength','length','size','type','name',),
deprecated=_DEPRECATED,
)

glGetUniformfvARB = platform.createExtensionFunction( 
'glGetUniformfvARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLhandleARB,constants.GLint,arrays.GLfloatArray,),
doc='glGetUniformfvARB(GLhandleARB(programObj), GLint(location), GLfloatArray(params)) -> None',
argNames=('programObj','location','params',),
deprecated=_DEPRECATED,
)

glGetUniformivARB = platform.createExtensionFunction( 
'glGetUniformivARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLhandleARB,constants.GLint,arrays.GLintArray,),
doc='glGetUniformivARB(GLhandleARB(programObj), GLint(location), GLintArray(params)) -> None',
argNames=('programObj','location','params',),
deprecated=_DEPRECATED,
)

glGetShaderSourceARB = platform.createExtensionFunction( 
'glGetShaderSourceARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLhandleARB,constants.GLsizei,arrays.GLsizeiArray,arrays.GLcharARBArray,),
doc='glGetShaderSourceARB(GLhandleARB(obj), GLsizei(maxLength), GLsizeiArray(length), GLcharARBArray(source)) -> None',
argNames=('obj','maxLength','length','source',),
deprecated=_DEPRECATED,
)


def glInitShaderObjectsARB():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



import pygame, sys
from pygame.locals import *
import OpenGL
OpenGL.USE_ACCELERATE = False
from OpenGL.GL import *
from OpenGL.GL.EXT.framebuffer_object import *
from OpenGL.platform import safeGetError
from OpenGL import error

def draw ():
    glClearColor(0.0,0.0,0.0,0.0)
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
    #draw stuff here
    pygame.display.flip()

pygame.init()
pygame.display.set_mode((512,512),OPENGL | DOUBLEBUF)

#setup a texture
tex = glGenTextures(1);
glBindTexture(GL_TEXTURE_2D, tex);
glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, 512, 512, 0, GL_RGBA,
GL_UNSIGNED_BYTE, None);
glBindTexture(GL_TEXTURE_2D, 0);

#setup teh fbo
fbo = glGenFramebuffersEXT(1)
glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, fbo)
glBindTexture(GL_TEXTURE_2D, tex)

#this call produces an error!

glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT,
GL_COLOR_ATTACHMENT0_EXT,GL_TEXTURE_2D, tex, 0)

while 1:
    event=pygame.event.poll ()
    if event.type is QUIT:
        sys.exit(0)
    draw()
